/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: Game.java - core
 * Version: 0.2.5
 * Last modified: 2008/10/10 19:51 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.core;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import net.reneknipschild.cheesecaskets.gui.GameComponent;
import net.reneknipschild.cheesecaskets.gui.MainWindow;
import net.reneknipschild.cheesecaskets.gui.SidebarComponent;
import net.reneknipschild.cheesecaskets.gui.WaitWindow;
import net.reneknipschild.cheesecaskets.tools.SoundPlayer;

public class Game {
	private Settings settings;

	private MainWindow mainWindow;

	private SidebarComponent sidebarComponent;

	private GameComponent gameComponent;
	
	private WaitWindow waitWindow;

	private boolean game;

	private SoundPlayer backgroundMusic;

	private Timer timeCounter;

	private int timeElapsed;

	public Game(Settings settings) {
		this.settings = settings;
	}

	public void transmitComponentInstances(MainWindow mainWindow, SidebarComponent sidebarComponent, GameComponent gameComponent) {
		this.mainWindow = mainWindow;
		this.sidebarComponent = sidebarComponent;
		this.gameComponent = gameComponent;
	}

	public void transmitWaitWindowInstance(WaitWindow waitWindow) {
		this.waitWindow = waitWindow;
	}
	
	public void startNewGame() {
		game = true;
		mainWindow.switchGameMode(true);
		sidebarComponent.switchGameMode(true);
		gameComponent.switchGameMode(true);
		if (Boolean.parseBoolean(settings.getSetting("sound-activated")) && Boolean.parseBoolean(settings.getSetting("sound-game")) && Boolean.parseBoolean(settings.getSetting("sound-gamebackground"))) {
			backgroundMusic = new SoundPlayer("Game.ogg", true);
			backgroundMusic.start();
		}
		timeCounter = new Timer();
		timeCounter.schedule(new TimeElapsedCounter(), 0, 1000);
		//waitWindow.dispose();
		waitWindow.toString();
	}

	class TimeElapsedCounter extends TimerTask {
		public void run() {
			sidebarComponent.updateTimeElapsed(new SimpleDateFormat("m:ss").format(new Date(timeElapsed++ * 1000)));
		}
	}

	public void exitCurrentGame() {
		game = false;
		if (backgroundMusic != null) {
			backgroundMusic.exit();
		}
		timeCounter.cancel();
		timeElapsed = 0;
		gameComponent.switchGameMode(false);
		sidebarComponent.switchGameMode(false);
		mainWindow.switchGameMode(false);
		// TODO If game not cancelled, show last game instead of default view
	}

	public boolean isGame() {
		return game;
	}
}