/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: Main.java - core
 * Version: 1.0.0
 * Last modified: 2008/07/23 23:44 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.core;

import net.reneknipschild.cheesecaskets.gui.MainWindow;
import net.reneknipschild.cheesecaskets.gui.SplashScreen;

public class Main {
	private static Settings settings;
	private static Game game;

	public static void main(String[] arguments) {
		checkJavaRuntimeVersion();
		loadSettings();
		loadGame();
		startGUI();
	}

	private static void checkJavaRuntimeVersion() {
		int requiredVersion = Integer.parseInt(Info.REQUIRED_JAVA_RUNTIME_VERSION.substring(2, 3));
		int currentVersion = Integer.parseInt(System.getProperty("java.runtime.version").substring(2, 3));
		if (currentVersion < requiredVersion) {
			System.err.println("Fatal error:\n\t" + Info.PROJECT_NAME + " requires Java Runtime Version " + Info.REQUIRED_JAVA_RUNTIME_VERSION + " or later.\n\tYou run Java Runtime Version " + System.getProperty("java.runtime.version").substring(0, 3) + ".\n\tYou need to upgrade if you want to run " + Info.PROJECT_NAME + ".");
			System.exit(1);
		}
	}

	private static void loadSettings() {
		settings = new Settings();
	}

	private static void loadGame() {
		game = new Game(settings);
	}

	private static void startGUI() {
		SplashScreen splash = new SplashScreen();
		splash.start();
		try {
			splash.join();
		} catch (InterruptedException exception) {
			exception.printStackTrace();
		}
		MainWindow window = new MainWindow(settings, game);
		window.start();
		try {
			window.join();
		} catch (InterruptedException exception) {
			exception.printStackTrace();
		}
		splash.dispose();
	}
}