/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: Settings.java - core
 * Version: 1.0.2
 * Last modified: 2008/10/02 16:10 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import net.reneknipschild.cheesecaskets.tools.BitmapLoader;
import net.reneknipschild.cheesecaskets.tools.FileAction;
import net.reneknipschild.cheesecaskets.tools.LookAndFeelSetter;

public class Settings {
	private File configuration;

	private File avatar;

	private File configurationDirectory;

	private Properties settings;

	private Properties localisation;

	public Settings() {
		settings = new Properties();

		configuration = new File(System.getProperty("user.home") + "/.cheesecaskets/settings.properties");
		avatar = new File(System.getProperty("user.home") + "/.cheesecaskets/avatar.image");
		configurationDirectory = new File(System.getProperty("user.home") + "/.cheesecaskets/");

		if (!configuration.exists() || !avatar.exists()) {
			if (!configurationDirectory.exists()) {
				if (!configurationDirectory.mkdir()) {
					showConfigCreationErrorAndExit();
				}
			} else if (configurationDirectory.isFile()) {
				showConfigCreationErrorAndExit();
			}
			try {
				configuration.createNewFile();
				avatar.createNewFile();
			} catch (IOException exception) {
				showConfigCreationErrorAndExit();
			}
			try {
				settings.load(new FileInputStream(System.getProperty("user.home") + "/.cheesecaskets/settings.properties"));
			} catch (FileNotFoundException exception) {
				exception.printStackTrace();
			} catch (IOException exception) {
				exception.printStackTrace();
			}
			settings.setProperty("window-x", "null");
			settings.setProperty("window-y", "null");
			settings.setProperty("window-width", "800");
			settings.setProperty("window-height", "600");
			settings.setProperty("window-maximized", "false");
			settings.setProperty("window-fullscreen", "false");
			settings.setProperty("window-menubar", "true");
			settings.setProperty("window-toolbar", "true");
			settings.setProperty("window-sidebar", "true");
			settings.setProperty("window-chat", "true");
			settings.setProperty("window-statusbar", "true");
			settings.setProperty("gui-language", "english");
			settings.setProperty("gui-theme", "black");
			settings.setProperty("avatar-taken", "false");
			settings.setProperty("sound-activated", "true");
			settings.setProperty("sound-gui", "true");
			settings.setProperty("sound-game", "true");
			settings.setProperty("sound-gameclick", "true");
			settings.setProperty("sound-gamechat", "true");
			settings.setProperty("sound-gamebackground", "false");
			settings.setProperty("sound-gameend", "true");
			settings.setProperty("colors-background", "404040");
			settings.setProperty("colors-gamebackground", "ffff99");
			settings.setProperty("colors-gameborder", "ff3333");
			settings.setProperty("colors-fieldsetbackground", "00c907");
			settings.setProperty("colors-fieldsetborder", "000000");
			settings.setProperty("colors-fieldbackground", "0099ff");
			settings.setProperty("colors-fieldborder", "ffffff");
			settings.setProperty("proxy-taken", "false");
			settings.setProperty("proxy-0host", "0.0.0.0");
			settings.setProperty("proxy-0port", "0");
			settings.setProperty("multiplayer-taken", "false");
			settings.setProperty("multiplayer-0player1", System.getProperty("user.name"));
			settings.setProperty("multiplayer-0player2", "");
			settings.setProperty("multiplayer-0player3", "");
			settings.setProperty("multiplayer-0player4", "");
			settings.setProperty("multiplayer-0player5", "");
			settings.setProperty("timelimit-taken", "false");
			settings.setProperty("timelimit-time", "0");
			settings.setProperty("server-taken", "false");
			settings.setProperty("server-0port", "0");
			settings.setProperty("server-0password", "");
			settings.setProperty("server-1host", "0.0.0.0");
			settings.setProperty("server-1port", "0");
			settings.setProperty("server-1password", "");
			saveSettings();
		} else {
			try {
				settings.load(new FileInputStream(System.getProperty("user.home") + "/.cheesecaskets/settings.properties"));
			} catch (FileNotFoundException exception) {
				exception.printStackTrace();
			} catch (IOException exception) {
				exception.printStackTrace();
			}
			try {
				settings.getProperty("window-x").equals(null);
				settings.getProperty("window-y").equals(null);
				settings.getProperty("window-width").equals(null);
				settings.getProperty("window-height").equals(null);
				settings.getProperty("window-maximized").equals(null);
				settings.getProperty("window-fullscreen").equals(null);
				settings.getProperty("window-menubar").equals(null);
				settings.getProperty("window-toolbar").equals(null);
				settings.getProperty("window-sidebar").equals(null);
				settings.getProperty("window-chat").equals(null);
				settings.getProperty("window-statusbar").equals(null);
				settings.getProperty("gui-language").equals(null);
				settings.getProperty("gui-theme").equals(null);
				settings.getProperty("avatar-taken").equals(null);
				settings.getProperty("sound-activated").equals(null);
				settings.getProperty("sound-gui").equals(null);
				settings.getProperty("sound-game").equals(null);
				settings.getProperty("sound-gameclick").equals(null);
				settings.getProperty("sound-gamechat").equals(null);
				settings.getProperty("sound-gamebackground").equals(null);
				settings.getProperty("sound-gameend").equals(null);
				settings.getProperty("colors-background").equals(null);
				settings.getProperty("colors-gamebackground").equals(null);
				settings.getProperty("colors-gameborder").equals(null);
				settings.getProperty("colors-fieldsetbackground").equals(null);
				settings.getProperty("colors-fieldsetborder").equals(null);
				settings.getProperty("colors-fieldbackground").equals(null);
				settings.getProperty("colors-fieldborder").equals(null);
				settings.getProperty("proxy-taken").equals(null);
				settings.getProperty("proxy-0host").equals(null);
				settings.getProperty("proxy-0port").equals(null);
				settings.getProperty("multiplayer-taken").equals(null);
				settings.getProperty("multiplayer-0player1").equals(null);
				settings.getProperty("multiplayer-0player2").equals(null);
				settings.getProperty("multiplayer-0player3").equals(null);
				settings.getProperty("multiplayer-0player4").equals(null);
				settings.getProperty("multiplayer-0player5").equals(null);
				settings.getProperty("timelimit-taken").equals(null);
				settings.getProperty("timelimit-time").equals(null);
				settings.getProperty("server-taken").equals(null);
				settings.getProperty("server-0port").equals(null);
				settings.getProperty("server-0password").equals(null);
				settings.getProperty("server-1host").equals(null);
				settings.getProperty("server-1port").equals(null);
				settings.getProperty("server-1password").equals(null);
			} catch (NullPointerException exception) {
				showConfigReadErrorAndExit();
			}
		}

		localisation = new Properties();
		try {
			if (getSetting("gui-language").equals("english")) {
				localisation.load(ClassLoader.getSystemResourceAsStream("net/reneknipschild/cheesecaskets/resources/languages/English.properties"));
			} else if (getSetting("gui-language").equals("french")) {
				localisation.load(ClassLoader.getSystemResourceAsStream("net/reneknipschild/cheesecaskets/resources/languages/French.properties"));
			} else if (getSetting("gui-language").equals("german")) {
				localisation.load(ClassLoader.getSystemResourceAsStream("net/reneknipschild/cheesecaskets/resources/languages/German.properties"));
			} else if (getSetting("gui-language").equals("latin")) {
				localisation.load(ClassLoader.getSystemResourceAsStream("net/reneknipschild/cheesecaskets/resources/languages/Latin.properties"));
			}
		} catch (FileNotFoundException exception) {
			exception.printStackTrace();
		} catch (IOException exception) {
			exception.printStackTrace();
		}

		boolean sound = false;
		if (Boolean.parseBoolean(getSetting("sound-activated")) && Boolean.parseBoolean(getSetting("sound-gui"))) {
			sound = true;
		}
		if (getSetting("gui-theme").equals("black")) {
			LookAndFeelSetter.setLookAndSound(LookAndFeelSetter.BLACK_LOOK_AND_FEEL, sound);
		} else if (getSetting("gui-theme").equals("silver")) {
			LookAndFeelSetter.setLookAndSound(LookAndFeelSetter.SILVER_LOOK_AND_FEEL, sound);
		} else if (getSetting("gui-theme").equals("green")) {
			LookAndFeelSetter.setLookAndSound(LookAndFeelSetter.GREEN_LOOK_AND_FEEL, sound);
		} else if (getSetting("gui-theme").equals("red")) {
			LookAndFeelSetter.setLookAndSound(LookAndFeelSetter.RED_LOOK_AND_FEEL, sound);
		} else if (getSetting("gui-theme").equals("blue")) {
			LookAndFeelSetter.setLookAndSound(LookAndFeelSetter.BLUE_LOOK_AND_FEEL, sound);
		} else {
			LookAndFeelSetter.setSound(sound);
		}
	}

	public void setSetting(String name, String value) {
		settings.setProperty(name, value);
	}

	public String getSetting(String name) {
		return settings.getProperty(name);
	}

	public void saveSettings() {
		try {
			settings.store(new FileOutputStream(System.getProperty("user.home") + "/.cheesecaskets/settings.properties"), null);
		} catch (FileNotFoundException exception) {
			showConfigStoreErrorAndExit();
		} catch (IOException exception) {
			exception.printStackTrace();
		}
	}

	public void setAvatar(String newAvatarPath) {
		try {
			FileAction.copy(new File(newAvatarPath), avatar);
		} catch (IOException exception) {
			exception.printStackTrace();
		}
	}

	public ImageIcon getAvatar() {
		if (Boolean.parseBoolean(getSetting("avatar-taken"))) {
			return new ImageIcon(new ImageIcon(System.getProperty("user.home") + "/.cheesecaskets/avatar.image").getImage().getScaledInstance(128, 128, 0));
		}
		return BitmapLoader.getIcon("User128x128.png");
	}

	public String getLocalisation(String name) {
		return localisation.getProperty(name);
	}

	private void showConfigCreationErrorAndExit() {
		JOptionPane.showMessageDialog(null, "Unable to create the CheeseCaskets-Configuration!\n\nMake sure that there is no file called \".cheesecaskets\" in your home directory\nand that CheeseCaskets has the permission to write to your directory.\nCheeseCaskets is not able to start since it is able to create the configuration.\n\nWill now exit.", "Critical Error", JOptionPane.ERROR_MESSAGE);
		System.exit(1);
	}

	private void showConfigReadErrorAndExit() {
		JOptionPane.showMessageDialog(null, "Unable to read the CheeseCaskets-Configuration!\n\nMake sure that you didn't affected the directory \".cheesecaskets\" in your home\ndirectory. If so, you can try to repair the configuration or just delete it: In\nlast case, CheeseCaskets will create a new configuration. But remember that all\nyour settings will be lost, then. CheeseCaskets is not able to start since it is\nable to read or recreate the configuration.\n\nWill now exit.", "Critical Error", JOptionPane.ERROR_MESSAGE);
		System.exit(1);
	}

	private void showConfigStoreErrorAndExit() {
		JOptionPane.showMessageDialog(null, "Unable to store the CheeseCaskets-Configuration!\n\nMake sure that you didn't affected the directory \".cheesecaskets\" in your home\ndirectory. But in any case, your just token settings, and, of course, all your\nsettings token before, are lost, now. Just try to restart CheeseCaskets: This\nerror should not occur again. But however if it comes up again, something goes\nwrong with the access to the CheeseCaskets-Configuration: Only CheeseCaskets\nshould access to it's configuration.\n\nWill now exit.", "Critical Error", JOptionPane.ERROR_MESSAGE);
		System.exit(1);
	}
}