/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: AboutWindow.java - gui
 * Version: 1.0.0
 * Last modified: 2008/07/23 23:44 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.WindowConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import net.reneknipschild.cheesecaskets.core.Info;
import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.gui.components.BitmapComponent;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;
import net.reneknipschild.cheesecaskets.tools.Browser;

public class AboutWindow {
	private JDialog window;

	private JTabbedPane tabpane;

	private JPanel[] tabs;

	private JEditorPane[] text;

	private String contributors;

	private String third_partys;

	public AboutWindow(final Settings settings, JFrame owner) {
		window = new JDialog(owner, settings.getLocalisation("about"), true);
		window.setMinimumSize(new Dimension(415, 415));
		window.setResizable(true);
		window.setLocationRelativeTo(owner);
		window.setIconImage(BitmapLoader.getImage("Icon.png"));
		window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		window.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				close();
			}
		});

		contributors = "<br><br><font face=\"sans-serif\" size=5><b>" + settings.getLocalisation("authors") + "</b></font>";
		for (int i = 0; i < Info.AUTHORS.length; i++) {
			for (int j = 0; j < Info.AUTHORS[i].length; j++) {
				switch (j) {
				case 0:
					contributors += "<br><br><b>" + Info.AUTHORS[i][j] + "</b>";
					break;
				case 1:
					contributors += "<br><a href=\"mailto:" + Info.AUTHORS[i][j] + "\">" + Info.AUTHORS[i][j] + "</a>";
					break;
				}
			}
		}
		contributors += "<br><br><font face=\"sans-serif\" size=5><b>" + settings.getLocalisation("translators") + "</b></font>";
		for (int i = 0; i < Info.TRANSLATORS.length; i++) {
			for (int j = 0; j < Info.TRANSLATORS[i].length; j++) {
				switch (j) {
				case 0:
					contributors += "<br><br><b>" + Info.TRANSLATORS[i][j] + "</b>";
					break;
				case 1:
					contributors += "<br><a href=\"mailto:" + Info.TRANSLATORS[i][j] + "\">" + Info.TRANSLATORS[i][j] + "</a>";
					break;
				}
			}
		}
		contributors += "<br><br><font face=\"sans-serif\" size=5><b>" + settings.getLocalisation("musicians") + "</b></font>";
		for (int i = 0; i < Info.MUSICIANS.length; i++) {
			for (int j = 0; j < Info.MUSICIANS[i].length; j++) {
				switch (j) {
				case 0:
					contributors += "<br><br><b>" + Info.MUSICIANS[i][j] + "</b>";
					break;
				case 1:
					contributors += "<br><a href=\"mailto:" + Info.MUSICIANS[i][j] + "\">" + Info.MUSICIANS[i][j] + "</a>";
					break;
				}
			}
		}

		third_partys = "";
		for (int i = 0; i < Info.THIRD_PARTYS.length; i++) {
			for (int j = 0; j < Info.THIRD_PARTYS[i].length; j++) {
				switch (j) {
				case 0:
					third_partys += "<br><br><b>" + Info.THIRD_PARTYS[i][j] + "</b>";
					break;
				case 1:
					third_partys += "<br><a href=\"" + Info.THIRD_PARTYS[i][j] + "\">" + settings.getLocalisation("show_license") + "</a>";
					break;
				}
			}
		}

		JPanel header = new JPanel(new BorderLayout());
		header.setBackground(new Color(187, 187, 187));
		header.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		header.add(new BitmapComponent("InfoHead.png"), BorderLayout.CENTER);
		window.add(header, BorderLayout.NORTH);

		JPanel frame = new JPanel(new BorderLayout());
		frame.setBorder(BorderFactory.createEmptyBorder(8, 8, 0, 8));

		tabpane = new JTabbedPane();
		tabs = new JPanel[4];
		text = new JEditorPane[4];
		tabs[0] = new JPanel(new BorderLayout());
		text[0] = new JEditorPane();
		tabs[1] = new JPanel(new BorderLayout());
		text[1] = new JEditorPane();
		tabs[2] = new JPanel(new BorderLayout());
		text[2] = new JEditorPane();
		tabs[3] = new JPanel(new BorderLayout());
		text[3] = new JEditorPane();

		for (int i = 0; i < text.length; i++) {
			text[i].setContentType("text/html");
			text[i].setForeground(Color.BLACK);
			text[i].setBackground(Color.WHITE);
			text[i].setEditable(false);
			text[i].setFocusable(true);
			text[i].addHyperlinkListener(new HyperlinkListener() {
				public void hyperlinkUpdate(HyperlinkEvent event) {
					if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
						String url = event.getURL().toString();
						if (url.substring(7, 15).equals("document")) {
							new DocumentWindow(window, settings, url);
						} else {
							Browser.open(url);
						}
					}
				}
			});
			tabs[i].add(new JScrollPane(text[i], ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
		}
		text[0].setText("<html><font face=\"sans-serif\" size=6><b>" + Info.PROJECT_NAME + "</b></font><font face=\"sans-serif\" size=4><br><br>" + settings.getLocalisation("info__remark") + "<br><br><b>Version: " + Info.VERSION + "</b>" + "<br><br><b>Build id: " + Info.BUILD_ID + "</b>" + "<br><br>" + settings.getLocalisation("license__remark") + "<br><br><center><img border=0 src=\"" + BitmapLoader.getIcon("Distributor.png") + "\"><br><b>" + Info.RK_COPYRIGHT + "</b><br><a href=\"" + Info.RK_URI + "\">" + Info.RK_URI + "</a></center></font></html>");
		text[0].setCaretPosition(0);
		text[1].setText("<html><font face=\"sans-serif\" size=6><b>" + settings.getLocalisation("contributors") + "</b></font><font face=\"sans-serif\" size=4><br><br>" + settings.getLocalisation("contributors__remark") + contributors + "</font></html>");
		text[1].setCaretPosition(0);
		try {
			text[2].setPage(ClassLoader.getSystemResource("net/reneknipschild/cheesecaskets/resources/documents/gpl-3.0-standalone.html"));
		} catch (IOException exception) {
			exception.printStackTrace();
		}
		text[2].setCaretPosition(0);
		text[3].setText("<html><font face=\"sans-serif\" size=6><b>" + settings.getLocalisation("third_parties") + "</b></font><br><br><font face=\"sans-serif\" size=4>" + settings.getLocalisation("third_parties__remark") + third_partys + "</font></html>");
		text[3].setCaretPosition(0);

		tabpane.addTab(settings.getLocalisation("info"), tabs[0]);
		tabpane.addTab(settings.getLocalisation("contributors"), tabs[1]);
		tabpane.addTab(settings.getLocalisation("license"), tabs[2]);
		tabpane.addTab(settings.getLocalisation("third_parties"), tabs[3]);
		frame.add(tabpane, BorderLayout.CENTER);
		window.add(frame, BorderLayout.CENTER);

		JPanel bottom = new JPanel();
		JButton close = new JButton(settings.getLocalisation("close"));
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
		bottom.add(close);
		window.add(bottom, BorderLayout.SOUTH);

		initializeShortCuts();
		window.setVisible(true);
	}

	private void close() {
		window.dispose();
	}

	private void initializeShortCuts() {
		InputMap inputmap = window.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionmap = window.getRootPane().getActionMap();
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "close");
		actionmap.put("close", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
	}
}