/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: ChatComponent.java - gui
 * Version: 0.5.4
 * Last modified: 2008/10/03 15:00 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;

import net.reneknipschild.cheesecaskets.core.Info;
import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;


public class ChatComponent extends JPanel {
	private static final long serialVersionUID = 1L;

	private JEditorPane content;

	private JPanel bottom;

	private JTextField input;

	private JButton send;

	private JPopupMenu menu;

	private JMenuItem copy;

	private JMenuItem selectall;

	private JMenuItem clear;

	public ChatComponent(Settings settings) {
		setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
		setLayout(new BorderLayout(8, 8));

		content = new JEditorPane();
		bottom = new JPanel(new BorderLayout(8, 8));
		input = new JTextField();
		send = new JButton(settings.getLocalisation("send"), BitmapLoader.getIcon("Send16x16.png"));

		content.setContentType("text/plain");
		content.setForeground(Color.BLACK);
		content.setBackground(Color.WHITE);
		content.setFont(content.getFont().deriveFont(11f));
		content.setEditable(false);
		content.setFocusable(true);
		addText("System", "Welcome to CheeseCaskets v" + Info.VERSION + "!");
		send.setHorizontalTextPosition(SwingConstants.LEFT);
		send.setEnabled(false);

		content.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent event) {
				showPopupMenu(event);
			}

			public void mouseEntered(MouseEvent event) {

			}

			public void mouseExited(MouseEvent event) {

			}

			public void mousePressed(MouseEvent event) {
				showPopupMenu(event);
			}

			public void mouseReleased(MouseEvent event) {
				showPopupMenu(event);
			}
		});

		input.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent event) {
				if (event.getKeyCode() == KeyEvent.VK_ENTER) {
					addText("User", input.getText());
				}
			}

			public void keyReleased(KeyEvent event) {
				if (!input.getText().equals("")) {
					send.setEnabled(true);
				} else {
					send.setEnabled(false);
				}
			}

			public void keyTyped(KeyEvent event) {

			}
		});

		send.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				addText("User", input.getText());
			}
		});

		add(new JLabel("<html><b>" + settings.getLocalisation("chat") + "</b></html>"), BorderLayout.NORTH);
		add(new JScrollPane(content, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
		bottom.add(input, BorderLayout.CENTER);
		bottom.add(send, BorderLayout.EAST);
		add(bottom, BorderLayout.SOUTH);

		initializeMenu(settings);
	}

	private void initializeMenu(Settings settings) {
		menu = new JPopupMenu();
		copy = new JMenuItem(settings.getLocalisation("copy"), BitmapLoader.getIcon("Copy16x16.png"));
		selectall = new JMenuItem(settings.getLocalisation("select_all"), BitmapLoader.getIcon("SelectAll16x16.png"));
		clear = new JMenuItem(settings.getLocalisation("clear"), BitmapLoader.getIcon("Clear16x16.png"));

		copy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				content.copy();
			}
		});

		selectall.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				content.grabFocus();
				content.selectAll();
			}
		});

		clear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				content.setText("");
			}
		});

		menu.add(copy);
		menu.add(selectall);
		menu.addSeparator();
		menu.add(clear);
	}

	private void showPopupMenu(MouseEvent event) {
		if (event.isPopupTrigger()) {
			menu.show(event.getComponent(), event.getX(), event.getY());
		}
	}

	public void addText(String user, String text) {
		if (!text.equals("")) {
			String newText = content.getText() + "<" + user + "> " + text + "\n";
			content.setText(newText);
			input.setText("");
			input.requestFocusInWindow();
			send.setEnabled(false);
		}
	}
}