/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: DocumentWindow.java - gui
 * Version: 1.0.0
 * Last modified: 2008/10/03 14:36 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.WindowConstants;

import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;

public class DocumentWindow {
	private JDialog window;

	private JPanel frame;

	private JEditorPane text;

	private JButton close;

	public DocumentWindow(JDialog owner, Settings settings, String url) {
		window = new JDialog(owner, true);
		window.setMinimumSize(new Dimension(500, 300));
		window.setResizable(true);
		window.setLocationRelativeTo(owner);
		window.setIconImage(BitmapLoader.getImage("Icon.png"));
		window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		window.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				close();
			}
		});
		frame = new JPanel(new BorderLayout());
		frame.setBorder(BorderFactory.createEmptyBorder(8, 8, 0, 8));
		text = new JEditorPane();
		try {
			text.setPage(ClassLoader.getSystemResource("net/reneknipschild/cheesecaskets/resources/documents/" + url.substring(16, url.length())));
		} catch (IOException exception) {
			exception.printStackTrace();
		}
		text.setForeground(Color.BLACK);
		text.setBackground(Color.WHITE);
		text.setEditable(false);
		text.setFocusable(true);
		text.setCaretPosition(0);
		JPanel bottom = new JPanel();
		close = new JButton(settings.getLocalisation("close"));
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
		bottom.add(close);
		frame.add(new JScrollPane(text, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED), BorderLayout.CENTER);
		frame.add(bottom, BorderLayout.SOUTH);
		window.add(frame, BorderLayout.CENTER);
		initializeShortCuts();
		window.setVisible(true);
	}

	private void close() {
		window.dispose();
	}

	private void initializeShortCuts() {
		InputMap inputmap = window.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionmap = window.getRootPane().getActionMap();
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "close");
		actionmap.put("close", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
	}
}