/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: EndgameWindow.java - gui
 * Version: 1.0.0
 * Last modified: 2008/07/23 23:44 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;

import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.gui.components.AvatarViewer;
import net.reneknipschild.cheesecaskets.gui.layouts.ComfortGridBagLayout;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;
import net.reneknipschild.cheesecaskets.tools.SoundPlayer;

public class EndgameWindow {
	private JDialog window;

	private JTabbedPane tabpane;

	private JPanel[] tabs;

	private ComfortGridBagLayout layout[];

	private JLabel resultIconLeft;

	private JLabel resultHeadline;

	private JLabel resultIconRight;

	private JLabel resultMessage;

	private JLabel detailsIcon;

	private JLabel detailsHeadline;

	private JLabel detailsListingWinnerLabel;

	private AvatarViewer detailsListingWinner;

	private JLabel detailsListingPlayersLabel;

	private JLabel detailsListingPlayers;

	private JLabel detailsListingTimeLabel;

	private JLabel detailsListingTime;

	private JButton close;

	private SoundPlayer backgroundSound;

	public EndgameWindow(final Settings settings, final JFrame owner, boolean winner, Icon winnerAvatar, String[] gameResults) {
		if (Boolean.parseBoolean(settings.getSetting("sound-activated")) && Boolean.parseBoolean(settings.getSetting("sound-game")) && Boolean.parseBoolean(settings.getSetting("sound-gameend"))) {
			String file = "";
			if (winner) {
				file = "Winner.ogg";
			} else {
				file = "Loser.ogg";
			}
			backgroundSound = new SoundPlayer(file, false);
			backgroundSound.start();
		}
		window = new JDialog(owner, settings.getLocalisation("end_game"), true);
		window.setMinimumSize(new Dimension(600, 400));
		window.setResizable(true);
		window.setLocationRelativeTo(owner);
		window.setIconImage(BitmapLoader.getImage("Icon.png"));
		window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		window.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				close();
			}
		});

		layout = new ComfortGridBagLayout[3];
		layout[0] = new ComfortGridBagLayout();
		layout[1] = new ComfortGridBagLayout();

		JPanel frame = new JPanel(new BorderLayout());
		frame.setBorder(BorderFactory.createEmptyBorder(8, 8, 0, 8));
		tabpane = new JTabbedPane();
		tabs = new JPanel[2];

		tabs[0] = new JPanel(layout[0]);

		tabs[0].setBackground(Color.WHITE);

		String icon;
		String headline;
		String remark;
		if (winner) {
			icon = "Winner128x128.png";
			headline = "win";
			remark = "win__remark";
		} else {
			icon = "Loser128x128.png";
			headline = "lose";
			remark = "lose__remark";
		}

		resultIconLeft = new JLabel(BitmapLoader.getIcon(icon));
		resultHeadline = new JLabel(settings.getLocalisation(headline));
		resultIconRight = new JLabel(BitmapLoader.getIcon(icon));
		resultMessage = new JLabel(settings.getLocalisation(remark));

		resultHeadline.setFont(resultHeadline.getFont().deriveFont(Font.BOLD, 42f));
		resultHeadline.setHorizontalAlignment(SwingConstants.CENTER);
		resultMessage.setFont(resultMessage.getFont().deriveFont(Font.ITALIC, 16f));
		resultMessage.setHorizontalAlignment(SwingConstants.CENTER);

		layout[0].createGridBagConstraints(resultIconLeft, 0, 0, 1, 1, GridBagConstraints.WEST, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(resultHeadline, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(resultIconRight, 2, 0, 1, 1, GridBagConstraints.EAST, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(resultMessage, 0, 1, 3, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		tabs[0].add(resultIconLeft);
		tabs[0].add(resultHeadline);
		tabs[0].add(resultIconRight);
		tabs[0].add(resultMessage);

		tabpane.addTab(settings.getLocalisation("result"), tabs[0]);

		tabs[1] = new JPanel(layout[1]);

		detailsIcon = new JLabel(BitmapLoader.getIcon("Details72x72.png"));
		detailsHeadline = new JLabel(settings.getLocalisation("details"));
		detailsListingWinnerLabel = new JLabel(settings.getLocalisation("winner") + ":");
		detailsListingWinner = new AvatarViewer(winnerAvatar, gameResults[0]);
		detailsListingPlayersLabel = new JLabel(settings.getLocalisation("players") + ":");
		detailsListingPlayers = new JLabel("<html>1. " + gameResults[0] + " <font color=\"#888888\"><i>(" + gameResults[1] + " " + settings.getLocalisation("moves") + ")</i></font><br>2. " + gameResults[2] + " <font color=\"#888888\"><i>(" + gameResults[3] + " " + settings.getLocalisation("moves") + ")</i></font><br>3. " + gameResults[4] + " <font color=\"#888888\"><i>(" + gameResults[5] + " " + settings.getLocalisation("moves") + ")</i></font><br>4. " + gameResults[6] + " <font color=\"#888888\"><i>(" + gameResults[7] + " " + settings.getLocalisation("moves") + ")</i></font><br>5. " + gameResults[8] + " <font color=\"#888888\"><i>(" + gameResults[9] + " " + settings.getLocalisation("moves") + ")</i></font><br></html>");
		detailsListingTimeLabel = new JLabel(settings.getLocalisation("time") + ":");
		detailsListingTime = new JLabel("<html>" + settings.getLocalisation("elapsed") + ": " + gameResults[10] + "<br>" + settings.getLocalisation("remaining") + ": " + gameResults[11] + "</html>");

		detailsHeadline.setFont(detailsHeadline.getFont().deriveFont(Font.BOLD, 24f));
		detailsListingWinnerLabel.setFont(detailsListingWinnerLabel.getFont().deriveFont(Font.BOLD));
		detailsListingPlayersLabel.setFont(detailsListingPlayersLabel.getFont().deriveFont(Font.BOLD));
		detailsListingTimeLabel.setFont(detailsListingTimeLabel.getFont().deriveFont(Font.BOLD));

		layout[1].createGridBagConstraints(detailsIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(detailsHeadline, 1, 0, 4, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(detailsListingWinnerLabel, 1, 1, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(detailsListingWinner, 2, 1, 1, 2, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(detailsListingPlayersLabel, 3, 1, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(detailsListingPlayers, 4, 1, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(detailsListingTimeLabel, 3, 2, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(detailsListingTime, 4, 2, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		tabs[1].add(detailsIcon);
		tabs[1].add(detailsHeadline);
		tabs[1].add(detailsListingWinnerLabel);
		tabs[1].add(detailsListingWinner);
		tabs[1].add(detailsListingPlayersLabel);
		tabs[1].add(detailsListingPlayers);
		tabs[1].add(detailsListingTimeLabel);
		tabs[1].add(detailsListingTime);

		tabpane.addTab(settings.getLocalisation("details"), tabs[1]);

		frame.add(tabpane, BorderLayout.CENTER);
		window.add(frame, BorderLayout.CENTER);

		window.add(frame, BorderLayout.CENTER);

		JPanel bottom = new JPanel();
		close = new JButton(settings.getLocalisation("close"));
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
		bottom.add(close);
		window.add(bottom, BorderLayout.SOUTH);

		initializeShortCuts();
		window.setVisible(true);
	}

	private void close() {
		window.dispose();
		if (backgroundSound != null) {
			backgroundSound.exit();
		}
	}

	private void initializeShortCuts() {
		InputMap inputmap = window.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionmap = window.getRootPane().getActionMap();
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "close");
		actionmap.put("close", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
	}
}