/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: GameComponent.java - gui
 * Version: 1.0.0
 * Last modified: 2008/07/23 23:44 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.tools.ColorConverter;

public class GameComponent extends JPanel {
	private static final long serialVersionUID = 1L;

	private Settings settings;

	private CardLayout layout;

	private JLabel placeholder;

	private JPanel frame;

	private JPanel holder;

	private JPanel game;

	private JPanel holder0;

	private JPanel holder1;

	private JPanel area[][];

	private JPanel field[][][][];

	private boolean gameIsVisible;

	public GameComponent(Settings settings) {
		this.settings = settings;
		rebuildUI();
	}

	private JPanel getField(Object source) {
		for (int i = 0; i < 5; i++) {
			for (int j = 0; j < 5; j++) {
				for (int k = 0; k < 3; k++) {
					for (int l = 0; l < 3; l++) {
						if (field[i][j][k][l].equals(source)) {
							return field[i][j][k][l];
						}
					}
				}
			}
		}
		return null;
	}

	private void hover(Object source) {
		getField(source).setBackground(Color.BLACK);
	}

	public void switchGameMode(boolean on) {
		rebuildUI();
		if (!gameIsVisible && on || gameIsVisible && !on) {
			layout.next(frame);
		}
		gameIsVisible = on;
	}

	private void rebuildUI() {
		removeAll();
		gameIsVisible = false;

		setLayout(new BorderLayout());
		setBackground(Color.WHITE);

		layout = new CardLayout();
		frame = new JPanel(layout);
		frame.setForeground(Color.BLACK);
		frame.setBackground(Color.WHITE);
		placeholder = new JLabel(settings.getLocalisation("new_game__remark"), SwingConstants.CENTER);
		placeholder.setForeground(Color.BLACK);
		holder = new JPanel(new BorderLayout());
		game = new JPanel(new GridLayout(5, 5));
		area = new JPanel[5][5];
		field = new JPanel[5][5][3][3];

		for (int i = 0; i < 5; i++) {
			for (int j = 0; j < 5; j++) {
				holder0 = new JPanel(new GridLayout());
				holder0.setBorder(BorderFactory.createLineBorder(ColorConverter.getColorFromHexString(settings.getSetting("colors-gamebackground")), 5));
				area[i][j] = new JPanel(new GridLayout(3, 3));
				area[i][j].setBorder(BorderFactory.createLineBorder(ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldsetborder"))));
				area[i][j].setBackground(ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldsetbackground")));
				for (int k = 0; k < 3; k++) {
					for (int l = 0; l < 3; l++) {
						holder1 = new JPanel(new GridLayout());
						holder1.setBorder(BorderFactory.createLineBorder(ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldsetbackground")), 2));
						field[i][j][k][l] = new JPanel();
						field[i][j][k][l].setBorder(BorderFactory.createLineBorder(ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldborder"))));
						field[i][j][k][l].setBackground(ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldbackground")));
						field[i][j][k][l].addMouseListener(new MouseListener() {
							public void mouseClicked(MouseEvent event) {

							}

							public void mouseEntered(MouseEvent event) {
								hover(event.getSource());
							}

							public void mouseExited(MouseEvent event) {

							}

							public void mousePressed(MouseEvent event) {

							}

							public void mouseReleased(MouseEvent event) {

							}
						});
						holder1.add(field[i][j][k][l]);
						area[i][j].add(holder1);
					}
				}
				holder0.add(area[i][j]);
				game.add(holder0);
			}
		}

		holder.setBorder(BorderFactory.createLineBorder(ColorConverter.getColorFromHexString(settings.getSetting("colors-background")), 10));
		game.setBorder(BorderFactory.createLineBorder(ColorConverter.getColorFromHexString(settings.getSetting("colors-gameborder")), 5));
		game.setBackground(ColorConverter.getColorFromHexString(settings.getSetting("colors-gamebackground")));

		holder.add(game, BorderLayout.CENTER);

		frame.add(holder, BorderLayout.CENTER);
		frame.add(placeholder, BorderLayout.CENTER);

		add(frame, BorderLayout.CENTER);
		layout.next(frame);
	}

	public Dimension getPreferredSize() {
		return new Dimension(500, 300);
	}
}