/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: MainWindow.java - gui
 * Version: 2.0.2
 * Last modified: 2008/10/03 00:58 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

import net.reneknipschild.cheesecaskets.core.Game;
import net.reneknipschild.cheesecaskets.core.Info;
import net.reneknipschild.cheesecaskets.core.Quit;
import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;
import net.reneknipschild.cheesecaskets.tools.Browser;

public class MainWindow extends Thread {
	private Settings settings;

	private Game game;

	private JFrame window;

	private JMenuBar menu;

	private JMenu menus[];

	private JMenuItem menuitems[][];

	private JCheckBoxMenuItem checkmenuitems[][];

	private JToolBar toolbar;

	private JButton[] toolbaritems;

	private JToggleButton[] toolbartoggleitems;

	private JPanel frame;

	private GameComponent gaming;

	private SidebarComponent sidebar;

	private ChatComponent chat;

	private JSplitPane[] splitter;

	private JToolBar statusbar;

	private JLabel statusbarText;

	private Dimension windowSize;

	private Point windowLocation;

	private boolean windowMaximized;

	private boolean isFullScreen;

	public MainWindow(Settings settings, Game game) {
		this.settings = settings;
		this.game = game;
	}

	public void run() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		UIManager.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				SwingUtilities.updateComponentTreeUI(window);
				window.repaint();
			}
		});
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				setName("MainWindow");
				windowSize = new Dimension();
				windowLocation = new Point();

				window = new JFrame(Info.PROJECT_NAME);

				window.setSize(new Dimension(Integer.parseInt(settings.getSetting("window-width")), Integer.parseInt(settings.getSetting("window-height"))));
				window.setMinimumSize(new Dimension(820, 680));
				if (settings.getSetting("window-x").equals("null") || settings.getSetting("window-y").equals("null")) {
					window.setLocationRelativeTo(null);
				} else {
					window.setLocation(new Point(Integer.parseInt(settings.getSetting("window-x")), Integer.parseInt(settings.getSetting("window-y"))));
				}
				window.setIconImage(BitmapLoader.getImage("Icon.png"));
				window.setLayout(new BorderLayout());
				window.setJMenuBar(menu());
				window.add(toolbar(), BorderLayout.NORTH);
				window.add(frame(), BorderLayout.CENTER);
				window.add(statusbar(), BorderLayout.SOUTH);
				if (!Boolean.parseBoolean(settings.getSetting("window-menubar"))) {
					setMenu();
				}
				if (!Boolean.parseBoolean(settings.getSetting("window-toolbar"))) {
					setToolbar();
				}
				if (!Boolean.parseBoolean(settings.getSetting("window-sidebar"))) {
					setSidebar();
				}
				if (!Boolean.parseBoolean(settings.getSetting("window-chat"))) {
					setChat();
				}
				if (!Boolean.parseBoolean(settings.getSetting("window-statusbar"))) {
					setStatusbar();
				}
				window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
				window.addWindowListener(new WindowAdapter() {
					public void windowClosing(WindowEvent event) {
						new Quit(settings, game, MainWindow.this, window);
					}
				});
				initializeShortCuts();
				window.setVisible(true);
				if (Boolean.parseBoolean(settings.getSetting("window-maximized"))) {
					setMaximized();
				}
				if (Boolean.parseBoolean(settings.getSetting("window-fullscreen"))) {
					setFullScreen();
				}
				window.addComponentListener(new ComponentListener() {
					public void componentHidden(ComponentEvent event) {

					}

					public void componentMoved(ComponentEvent event) {

					}

					public void componentResized(ComponentEvent event) {
						updateDividers();
					}

					public void componentShown(ComponentEvent event) {

					}
				});
				splitter[1].setDividerLocation(splitter[1].getWidth() - 285);
				splitter[0].setDividerLocation(splitter[0].getHeight() - 160);
				game.transmitComponentInstances(MainWindow.this, sidebar, gaming);
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						toolbaritems[0].requestFocusInWindow();
					}
				});
			}
		});
	}

	public void dispose() {
		window.dispose();
	}

	public int getWindowX() {
		return windowLocation.x;
	}

	public int getWindowY() {
		return windowLocation.y;
	}

	public int getWindowWidth() {
		return windowSize.width;
	}

	public int getWindowHeight() {
		return windowSize.height;
	}

	public void updateDividers() {
		splitter[1].setDividerLocation(splitter[1].getWidth() - 285);
		splitter[0].setDividerLocation(splitter[0].getHeight() - 160);
	}

	public boolean isFullScreen() {
		return isFullScreen;
	}

	public void setFullScreen() {
		if (!isFullScreen()) {
			if (window.getExtendedState() != Frame.MAXIMIZED_BOTH) {
				windowMaximized = false;
				windowSize = new Dimension(window.getWidth(), window.getHeight());
				windowLocation = window.getLocation();
			} else {
				windowMaximized = true;
				window.setExtendedState(Frame.NORMAL);
				// TODO Following try-catch-statement is really botch!
				try {
					Thread.sleep(1000);
				} catch (InterruptedException exception) {
					exception.printStackTrace();
				}
				windowSize = new Dimension(window.getWidth(), window.getHeight());
				windowLocation = window.getLocation();
			}
			window.dispose();
			window.setLocation(new Point(0, 0));
			window.setSize(Toolkit.getDefaultToolkit().getScreenSize());
			window.setResizable(false);
			checkmenuitems[1][5].setSelected(true);
			toolbartoggleitems[2].setSelected(true);
			window.setVisible(true);
			isFullScreen = true;
		} else {
			window.dispose();
			window.setResizable(true);
			window.setSize(windowSize);
			window.setLocation(windowLocation);
			checkmenuitems[1][5].setSelected(false);
			toolbartoggleitems[2].setSelected(false);
			window.setVisible(true);
			if (windowMaximized) {
				window.setExtendedState(Frame.MAXIMIZED_BOTH);
			}
			isFullScreen = false;
		}
		window.repaint();
	}

	public boolean isMaximized() {
		return windowMaximized;
	}

	public void setMaximized() {
		windowSize = new Dimension(window.getWidth(), window.getHeight());
		windowLocation = window.getLocation();
		window.setExtendedState(Frame.MAXIMIZED_BOTH);
	}

	public boolean isMenu() {
		if (menu.isVisible()) {
			return true;
		} else {
			return false;
		}
	}

	public void setMenu() {
		if (!isMenu()) {
			menu.setVisible(true);
			checkmenuitems[1][0].setSelected(true);
		} else {
			menu.setVisible(false);
			checkmenuitems[1][0].setSelected(false);
		}
	}

	public boolean isToolbar() {
		if (toolbar.isVisible()) {
			return true;
		} else {
			return false;
		}
	}

	public void setToolbar() {
		if (!isToolbar()) {
			toolbar.setVisible(true);
			checkmenuitems[1][1].setSelected(true);
		} else {
			toolbar.setVisible(false);
			checkmenuitems[1][1].setSelected(false);
		}
	}

	public boolean isSidebar() {
		if (sidebar.isVisible()) {
			return true;
		} else {
			return false;
		}
	}

	public void setSidebar() {
		if (!isSidebar()) {
			sidebar.setVisible(true);
			updateDividers();
			checkmenuitems[1][2].setSelected(true);
		} else {
			sidebar.setVisible(false);
			splitter[1].setDividerLocation(splitter[1].getWidth());
			checkmenuitems[1][2].setSelected(false);
		}
	}

	public boolean isChat() {
		if (chat.isVisible()) {
			return true;
		} else {
			return false;
		}
	}

	public void setChat() {
		if (!isChat()) {
			chat.setVisible(true);
			updateDividers();
			checkmenuitems[1][3].setSelected(true);
		} else {
			chat.setVisible(false);
			splitter[0].setDividerLocation(splitter[0].getHeight());
			checkmenuitems[1][3].setSelected(false);
		}
	}

	public boolean isStatusbar() {
		if (statusbar.isVisible()) {
			return true;
		} else {
			return false;
		}
	}

	public void setStatusbar() {
		if (!isStatusbar()) {
			statusbar.setVisible(true);
			checkmenuitems[1][4].setSelected(true);
		} else {
			statusbar.setVisible(false);
			checkmenuitems[1][4].setSelected(false);
		}
	}

	public void setStatusbarText(boolean isCurrentGame) {
		if (isCurrentGame) {
			statusbarText.setText(settings.getLocalisation("new_game__remark"));
		} else if (!isCurrentGame) {
			statusbarText.setText(settings.getLocalisation("quit_game__remark"));
		}
	}

	public void switchGameMode(boolean on) {
		menuitems[0][0].setEnabled(!on);
		menus[2].setEnabled(!on);
		menuitems[0][1].setEnabled(on);
		toolbaritems[0].setEnabled(!on);
		toolbaritems[3].setEnabled(!on);
		toolbaritems[1].setEnabled(on);
		setStatusbarText(!on);
	}

	private void initializeShortCuts() {
		InputMap inputmap = window.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionmap = window.getRootPane().getActionMap();
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK), "new_game");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK), "quit_game");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0), "menubar");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0), "toolbar");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, ActionEvent.CTRL_MASK), "sidebar");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, ActionEvent.CTRL_MASK), "chat");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F4, 0), "statusbar");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F11, 0), "fullscreen");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK), "settings");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0), "online_documentation");
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F8, ActionEvent.SHIFT_MASK), "endgame_window_tempcheat");
		actionmap.put("new_game", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				if (!game.isGame()) {
					new NewgameWindow(settings, game, MainWindow.this, gaming, window);
				}
			}
		});
		actionmap.put("quit_game", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				if (game.isGame()) {
					game.exitCurrentGame();
				}
			}
		});
		actionmap.put("menubar", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				setMenu();
			}
		});
		actionmap.put("toolbar", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				setToolbar();
			}
		});
		actionmap.put("sidebar", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				setSidebar();
			}
		});
		actionmap.put("chat", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				setChat();
			}
		});
		actionmap.put("statusbar", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				setStatusbar();
			}
		});
		actionmap.put("fullscreen", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				setFullScreen();
			}
		});
		actionmap.put("settings", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				if (!game.isGame()) {
					new SettingsWindow(settings, window);
				}
			}
		});
		actionmap.put("online_documentation", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				Browser.open(Info.DOCUMENTATION_URI);
			}
		});
		actionmap.put("endgame_window_tempcheat", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				String[] results = { "DeeDee0815", "20", "Picard", "10", "EifX", "5", "Djinndrache", "2", "Yadis", "1", "2:15", "7:45" };
				new EndgameWindow(settings, window, true, settings.getAvatar(), results);
			}
		});
	}

	private JMenuBar menu() {
		menu = new JMenuBar();
		menus = new JMenu[4];
		menuitems = new JMenuItem[4][5];
		checkmenuitems = new JCheckBoxMenuItem[4][6];

		menus[0] = new JMenu(settings.getLocalisation("game"));
		menus[1] = new JMenu(settings.getLocalisation("view"));
		menus[2] = new JMenu(settings.getLocalisation("settings"));
		menus[3] = new JMenu(settings.getLocalisation("help"));

		menus[0].setMnemonic(settings.getLocalisation("game__mnemonic").toCharArray()[0]);
		menus[1].setMnemonic(settings.getLocalisation("view__mnemonic").toCharArray()[0]);
		menus[2].setMnemonic(settings.getLocalisation("settings__mnemonic").toCharArray()[0]);
		menus[3].setMnemonic(settings.getLocalisation("help__mnemonic").toCharArray()[0]);

		menuitems[0][0] = new JMenuItem(settings.getLocalisation("new_game"), BitmapLoader.getIcon("New16x16.png"));
		menuitems[0][1] = new JMenuItem(settings.getLocalisation("quit_game"), BitmapLoader.getIcon("Cancel16x16.png"));
		menuitems[0][2] = new JMenuItem(settings.getLocalisation("exit"), BitmapLoader.getIcon("Exit16x16.png"));
		checkmenuitems[1][0] = new JCheckBoxMenuItem(settings.getLocalisation("menubar"));
		checkmenuitems[1][1] = new JCheckBoxMenuItem(settings.getLocalisation("toolbar"));
		checkmenuitems[1][2] = new JCheckBoxMenuItem(settings.getLocalisation("sidebar"));
		checkmenuitems[1][3] = new JCheckBoxMenuItem(settings.getLocalisation("chat"));
		checkmenuitems[1][4] = new JCheckBoxMenuItem(settings.getLocalisation("statusbar"));
		checkmenuitems[1][5] = new JCheckBoxMenuItem(settings.getLocalisation("fullscreen"), BitmapLoader.getIcon("Fullscreen16x16.png"));
		menuitems[2][0] = new JMenuItem(settings.getLocalisation("settings"), BitmapLoader.getIcon("Settings16x16.png"));
		menuitems[3][0] = new JMenuItem(settings.getLocalisation("online_documentation"), BitmapLoader.getIcon("Help16x16.png"));
		menuitems[3][1] = new JMenuItem(settings.getLocalisation("home_page"), BitmapLoader.getIcon("Home16x16.png"));
		menuitems[3][2] = new JMenuItem(settings.getLocalisation("about"), BitmapLoader.getIcon("Info16x16.png"));

		menuitems[0][0].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				new NewgameWindow(settings, game, MainWindow.this, gaming, window);
			}
		});
		menuitems[0][0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		menuitems[0][1].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				game.exitCurrentGame();
			}
		});
		menuitems[0][1].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
		menuitems[0][1].setEnabled(false);
		menuitems[0][2].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				new Quit(settings, game, MainWindow.this, window);
			}
		});
		checkmenuitems[1][0].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setMenu();
			}
		});
		checkmenuitems[1][0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
		checkmenuitems[1][0].setSelected(true);
		checkmenuitems[1][1].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setToolbar();
			}
		});
		checkmenuitems[1][1].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0));
		checkmenuitems[1][1].setSelected(true);
		checkmenuitems[1][2].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setSidebar();
			}
		});
		checkmenuitems[1][2].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, ActionEvent.CTRL_MASK));
		checkmenuitems[1][2].setSelected(true);
		checkmenuitems[1][3].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setChat();
			}
		});
		checkmenuitems[1][3].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, ActionEvent.CTRL_MASK));
		checkmenuitems[1][3].setSelected(true);
		checkmenuitems[1][4].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setStatusbar();
			}
		});
		checkmenuitems[1][4].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, 0));
		checkmenuitems[1][4].setSelected(true);
		checkmenuitems[1][5].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setFullScreen();
			}
		});
		checkmenuitems[1][5].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F11, 0));
		menuitems[2][0].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				new SettingsWindow(settings, window);
			}
		});
		menuitems[2][0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		menuitems[3][0].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				Browser.open(Info.DOCUMENTATION_URI);
			}
		});
		menuitems[3][0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
		menuitems[3][1].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				Browser.open(Info.PROJECT_URI);
			}
		});
		menuitems[3][2].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				new AboutWindow(settings, window);
			}
		});

		menus[0].add(menuitems[0][0]);
		menus[0].add(menuitems[0][1]);
		menus[0].addSeparator();
		menus[0].add(menuitems[0][2]);
		menus[1].add(checkmenuitems[1][0]);
		menus[1].add(checkmenuitems[1][1]);
		menus[1].add(checkmenuitems[1][2]);
		menus[1].add(checkmenuitems[1][3]);
		menus[1].add(checkmenuitems[1][4]);
		menus[1].addSeparator();
		menus[1].add(checkmenuitems[1][5]);
		menus[2].add(menuitems[2][0]);
		menus[3].add(menuitems[3][0]);
		menus[3].add(menuitems[3][1]);
		menus[3].addSeparator();
		menus[3].add(menuitems[3][2]);

		for (int i = 0; i < menus.length; i++) {
			menu.add(menus[i]);
		}

		return menu;
	}

	private JToolBar toolbar() {
		toolbar = new JToolBar();
		toolbaritems = new JButton[5];
		toolbartoggleitems = new JToggleButton[5];

		toolbar.setFloatable(false);

		toolbaritems[0] = new JButton(settings.getLocalisation("new_game"), BitmapLoader.getIcon("New32x32.png"));
		toolbaritems[1] = new JButton(settings.getLocalisation("quit_game"), BitmapLoader.getIcon("Cancel32x32.png"));
		toolbartoggleitems[2] = new JToggleButton(settings.getLocalisation("fullscreen"), BitmapLoader.getIcon("Fullscreen32x32.png"));
		toolbaritems[3] = new JButton(settings.getLocalisation("settings"), BitmapLoader.getIcon("Settings32x32.png"));
		toolbaritems[4] = new JButton(settings.getLocalisation("exit"), BitmapLoader.getIcon("Exit32x32.png"));

		toolbaritems[0].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				new NewgameWindow(settings, game, MainWindow.this, gaming, window);
			}
		});
		toolbaritems[0].setHorizontalTextPosition(SwingConstants.CENTER);
		toolbaritems[0].setVerticalTextPosition(SwingConstants.BOTTOM);
		toolbaritems[1].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				game.exitCurrentGame();
			}
		});
		toolbaritems[1].setHorizontalTextPosition(SwingConstants.CENTER);
		toolbaritems[1].setVerticalTextPosition(SwingConstants.BOTTOM);
		toolbaritems[1].setEnabled(false);
		toolbartoggleitems[2].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setFullScreen();
			}
		});
		toolbartoggleitems[2].setHorizontalTextPosition(SwingConstants.CENTER);
		toolbartoggleitems[2].setVerticalTextPosition(SwingConstants.BOTTOM);
		toolbaritems[3].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				new SettingsWindow(settings, window);
			}
		});
		toolbaritems[3].setHorizontalTextPosition(SwingConstants.CENTER);
		toolbaritems[3].setVerticalTextPosition(SwingConstants.BOTTOM);
		toolbaritems[4].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				new Quit(settings, game, MainWindow.this, window);
			}
		});
		toolbaritems[4].setHorizontalTextPosition(SwingConstants.CENTER);
		toolbaritems[4].setVerticalTextPosition(SwingConstants.BOTTOM);

		toolbar.add(toolbaritems[0]);
		toolbar.add(toolbaritems[1]);
		toolbar.add(toolbartoggleitems[2]);
		toolbar.add(toolbaritems[3]);
		toolbar.add(toolbaritems[4]);

		return toolbar;
	}

	private JPanel frame() {
		frame = new JPanel(new BorderLayout());
		gaming = new GameComponent(settings);
		sidebar = new SidebarComponent(settings);
		chat = new ChatComponent(settings);
		splitter = new JSplitPane[2];
		splitter[1] = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, gaming, sidebar);
		splitter[0] = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, splitter[1], chat);

		sidebar.addComponentListener(new ComponentListener() {
			public void componentHidden(ComponentEvent event) {

			}

			public void componentMoved(ComponentEvent event) {

			}

			public void componentResized(ComponentEvent event) {
				updateDividers();
			}

			public void componentShown(ComponentEvent event) {

			}
		});

		chat.addComponentListener(new ComponentListener() {
			public void componentHidden(ComponentEvent event) {

			}

			public void componentMoved(ComponentEvent event) {

			}

			public void componentResized(ComponentEvent event) {
				updateDividers();
			}

			public void componentShown(ComponentEvent event) {

			}
		});

		frame.add(splitter[0], BorderLayout.CENTER);
		return frame;
	}

	private JToolBar statusbar() {
		statusbar = new JToolBar();
		statusbarText = new JLabel(settings.getLocalisation("new_game__remark"));
		statusbar.setFloatable(false);
		statusbar.add(statusbarText);
		return statusbar;
	}
}