/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: NewgameWindow.java - gui
 * Version: 1.0.1
 * Last modified: 2008/10/10 19:52 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;

import net.reneknipschild.cheesecaskets.core.Game;
import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.gui.components.HostSelector;
import net.reneknipschild.cheesecaskets.gui.components.TimeSelector;
import net.reneknipschild.cheesecaskets.gui.layouts.ComfortGridBagLayout;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;
import net.reneknipschild.cheesecaskets.tools.JPasswordFieldReader;

public class NewgameWindow {
	private JDialog window;

	private JTabbedPane tabpane;

	private JPanel[] tabs;

	private ComfortGridBagLayout layout[];

	private JLabel playerIcon;

	private JLabel playerHeadline;

	private ButtonGroup playerSelection;

	private JLabel playerSelectionLabel;

	private JRadioButton playerSelectionLocal;

	private JRadioButton playerSelectionNetwork;

	private JLabel playerSelectionNetworkPlayer1Label;
	
	private JTextField playerSelectionNetworkPlayer1;

	private JLabel playerSelectionNetworkPlayer2Label;
	
	private JTextField playerSelectionNetworkPlayer2;

	private JLabel playerSelectionNetworkPlayer3Label;
	
	private JTextField playerSelectionNetworkPlayer3;

	private JLabel playerSelectionNetworkPlayer4Label;
	
	private JTextField playerSelectionNetworkPlayer4;

	private JLabel playerSelectionNetworkPlayer5Label;
	
	private JTextField playerSelectionNetworkPlayer5;

	private JLabel timeIcon;

	private JLabel timeHeadline;

	private ButtonGroup timeSelection;

	private JLabel timeSelectionLabel;

	private JRadioButton timeSelectionNolimit;

	private JRadioButton timeSelectionLimit;

	private TimeSelector timeSelectionLimitTime;

	private JLabel serverIcon;

	private JLabel serverHeadline;

	private JLabel serverSelectionLabel;

	private ButtonGroup serverSelection;

	private JRadioButton serverSelectionNewserver;

	private JLabel serverSelectionNewserverPasswordLabel;
	
	private JPasswordField serverSelectionNewserverPassword;

	private JLabel serverSelectionNewserverPortLabel;
	
	private HostSelector serverSelectionNewserverPort;

	private JRadioButton serverSelectionServer;

	private JLabel serverSelectionServerSelectLabel;
	
	private HostSelector serverSelectionServerSelect;

	private JLabel serverSelectionServerPasswordLabel;
	
	private JPasswordField serverSelectionServerPassword;

	private JLabel serverSelectionInfo;

	private JButton okay;

	private JButton close;

	public NewgameWindow(final Settings settings, final Game game, final MainWindow mainWindow, final GameComponent gameArea, final JFrame owner) {
		window = new JDialog(owner, settings.getLocalisation("new_game"), true);
		window.setMinimumSize(new Dimension(520, 480));
		window.setResizable(true);
		window.setLocationRelativeTo(owner);
		window.setIconImage(BitmapLoader.getImage("Icon.png"));
		window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		window.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				close();
			}
		});

		layout = new ComfortGridBagLayout[3];
		layout[0] = new ComfortGridBagLayout();
		layout[1] = new ComfortGridBagLayout();
		layout[2] = new ComfortGridBagLayout();

		JPanel frame = new JPanel(new BorderLayout());
		frame.setBorder(BorderFactory.createEmptyBorder(8, 8, 0, 8));
		tabpane = new JTabbedPane();
		tabs = new JPanel[3];

		tabs[0] = new JPanel(layout[0]);

		playerIcon = new JLabel(BitmapLoader.getIcon("Players72x72.png"));
		playerHeadline = new JLabel(settings.getLocalisation("players"));
		playerSelection = new ButtonGroup();
		playerSelectionLabel = new JLabel(settings.getLocalisation("game_type__remark"));
		playerSelectionLocal = new JRadioButton(settings.getLocalisation("game_type_local__remark"));
		playerSelectionNetwork = new JRadioButton(settings.getLocalisation("game_type_network__remark"));
		playerSelectionNetworkPlayer1Label = new JLabel(settings.getLocalisation("your_nick") + ":");
		playerSelectionNetworkPlayer1 = new JTextField(settings.getSetting("multiplayer-0player1"));
		playerSelectionNetworkPlayer2Label = new JLabel(settings.getLocalisation("player") + "2:");
		playerSelectionNetworkPlayer2 = new JTextField(settings.getSetting("multiplayer-0player2"));
		playerSelectionNetworkPlayer3Label = new JLabel(settings.getLocalisation("player") + "3:");
		playerSelectionNetworkPlayer3 = new JTextField(settings.getSetting("multiplayer-0player3"));
		playerSelectionNetworkPlayer4Label = new JLabel(settings.getLocalisation("player") + "4:");
		playerSelectionNetworkPlayer4 = new JTextField(settings.getSetting("multiplayer-0player4"));
		playerSelectionNetworkPlayer5Label = new JLabel(settings.getLocalisation("player") + "5:");
		playerSelectionNetworkPlayer5 = new JTextField(settings.getSetting("multiplayer-0player5"));

		playerHeadline.setFont(playerHeadline.getFont().deriveFont(Font.BOLD, 24f));
		playerSelection.add(playerSelectionLocal);
		playerSelection.add(playerSelectionNetwork);
		if (Boolean.parseBoolean(settings.getSetting("multiplayer-taken"))) {
			playerSelectionNetwork.setSelected(true);
			playerSelectionNetworkPlayer1Label.setEnabled(true);
			playerSelectionNetworkPlayer1.setEnabled(true);
			playerSelectionNetworkPlayer2Label.setEnabled(true);
			playerSelectionNetworkPlayer2.setEnabled(true);
			playerSelectionNetworkPlayer3Label.setEnabled(true);
			playerSelectionNetworkPlayer3.setEnabled(true);
			playerSelectionNetworkPlayer4Label.setEnabled(true);
			playerSelectionNetworkPlayer4.setEnabled(true);
			playerSelectionNetworkPlayer5Label.setEnabled(true);
			playerSelectionNetworkPlayer5.setEnabled(true);
		} else {
			playerSelectionLocal.setSelected(true);
			playerSelectionNetworkPlayer1Label.setEnabled(false);
			playerSelectionNetworkPlayer1.setEnabled(false);
			playerSelectionNetworkPlayer2Label.setEnabled(false);
			playerSelectionNetworkPlayer2.setEnabled(false);
			playerSelectionNetworkPlayer3Label.setEnabled(false);
			playerSelectionNetworkPlayer3.setEnabled(false);
			playerSelectionNetworkPlayer4Label.setEnabled(false);
			playerSelectionNetworkPlayer4.setEnabled(false);
			playerSelectionNetworkPlayer5Label.setEnabled(false);
			playerSelectionNetworkPlayer5.setEnabled(false);
		}

		layout[0].createGridBagConstraints(playerIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerHeadline, 1, 0, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionLabel, 1, 1, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionLocal, 1, 2, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetwork, 1, 3, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer1Label, 1, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer1, 2, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer2Label, 1, 5, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer2, 2, 5, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer3Label, 1, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer3, 2, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer4Label, 1, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer4, 2, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer5Label, 1, 8, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(playerSelectionNetworkPlayer5, 2, 8, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		playerSelectionLocal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		playerSelectionNetwork.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		tabs[0].add(playerIcon);
		tabs[0].add(playerHeadline);
		tabs[0].add(playerSelectionLabel);
		tabs[0].add(playerSelectionLocal);
		tabs[0].add(playerSelectionNetwork);
		tabs[0].add(playerSelectionNetworkPlayer1Label);
		tabs[0].add(playerSelectionNetworkPlayer1);
		tabs[0].add(playerSelectionNetworkPlayer2Label);
		tabs[0].add(playerSelectionNetworkPlayer2);
		tabs[0].add(playerSelectionNetworkPlayer3Label);
		tabs[0].add(playerSelectionNetworkPlayer3);
		tabs[0].add(playerSelectionNetworkPlayer4Label);
		tabs[0].add(playerSelectionNetworkPlayer4);
		tabs[0].add(playerSelectionNetworkPlayer5Label);
		tabs[0].add(playerSelectionNetworkPlayer5);

		tabpane.addTab(settings.getLocalisation("type"), tabs[0]);

		tabs[1] = new JPanel(layout[1]);

		timeIcon = new JLabel(BitmapLoader.getIcon("Clock72x72.png"));
		timeHeadline = new JLabel(settings.getLocalisation("length"));
		timeSelection = new ButtonGroup();
		timeSelectionLabel = new JLabel(settings.getLocalisation("time_type__remark"));
		timeSelectionNolimit = new JRadioButton(settings.getLocalisation("time_type_nolimit__remark"));
		timeSelectionLimit = new JRadioButton(settings.getLocalisation("time_type_limit__remark"));
		timeSelectionLimitTime = new TimeSelector(settings.getLocalisation("time") + ":", 0, 3600);

		timeHeadline.setFont(timeHeadline.getFont().deriveFont(Font.BOLD, 24f));
		timeSelection.add(timeSelectionNolimit);
		timeSelection.add(timeSelectionLimit);
		if (Boolean.parseBoolean(settings.getSetting("timelimit-taken"))) {
			timeSelectionLimit.setSelected(true);
			timeSelectionLimitTime.setEnabled(true);
		} else {
			timeSelectionNolimit.setSelected(true);
			timeSelectionLimitTime.setEnabled(false);
		}
		timeSelectionLimitTime.setValue(Integer.parseInt(settings.getSetting("timelimit-time")));

		layout[1].createGridBagConstraints(timeIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(timeHeadline, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(timeSelectionLabel, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(timeSelectionNolimit, 1, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(timeSelectionLimit, 1, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(timeSelectionLimitTime, 1, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		timeSelectionNolimit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		timeSelectionLimit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		tabs[1].add(timeIcon);
		tabs[1].add(timeHeadline);
		tabs[1].add(timeSelectionLabel);
		tabs[1].add(timeSelectionNolimit);
		tabs[1].add(timeSelectionLimit);
		tabs[1].add(timeSelectionLimitTime);

		tabpane.addTab(settings.getLocalisation("time"), tabs[1]);

		tabs[2] = new JPanel(layout[2]);

		serverIcon = new JLabel(BitmapLoader.getIcon("Server72x72.png"));
		serverHeadline = new JLabel(settings.getLocalisation("server"));
		serverSelection = new ButtonGroup();
		serverSelectionLabel = new JLabel(settings.getLocalisation("server_type__remark"));
		serverSelectionNewserver = new JRadioButton(settings.getLocalisation("server_type_newserver__remark"));
		serverSelectionNewserverPortLabel = new JLabel(settings.getLocalisation("port") + ":");
		serverSelectionNewserverPort = new HostSelector(false, null, false, null, HostSelector.PORT_ONLY);
		serverSelectionNewserverPasswordLabel = new JLabel(settings.getLocalisation("password") + ":");
		serverSelectionNewserverPassword = new JPasswordField(settings.getSetting("server-0password"));
		serverSelectionServer = new JRadioButton(settings.getLocalisation("server_type_server__remark"));
		serverSelectionServerSelectLabel = new JLabel(settings.getLocalisation("host") + ":");
		serverSelectionServerSelect = new HostSelector(false, null, true, settings.getLocalisation("port") + ":", HostSelector.HOST_AND_PORT);
		serverSelectionServerPasswordLabel = new JLabel(settings.getLocalisation("password") + ":");
		serverSelectionServerPassword = new JPasswordField(settings.getSetting("server-1password"));
		serverSelectionInfo = new JLabel("<html><b>" + settings.getLocalisation("note") + ":</b> " + settings.getLocalisation("server_type_info__remark") + "</html>");

		serverHeadline.setFont(serverHeadline.getFont().deriveFont(Font.BOLD, 24f));
		serverSelection.add(serverSelectionNewserver);
		serverSelection.add(serverSelectionServer);
		if (Boolean.parseBoolean(settings.getSetting("server-taken"))) {
			serverSelectionServer.setSelected(true);
			serverSelectionServerSelectLabel.setEnabled(true);
			serverSelectionServerSelect.setEnabled(true);
			serverSelectionServerPasswordLabel.setEnabled(true);
			serverSelectionServerPassword.setEnabled(true);
			serverSelectionNewserverPortLabel.setEnabled(false);
			serverSelectionNewserverPort.setEnabled(false);
			serverSelectionNewserverPasswordLabel.setEnabled(false);
			serverSelectionNewserverPassword.setEnabled(false);
		} else {
			serverSelectionNewserver.setSelected(true);
			serverSelectionServerSelectLabel.setEnabled(false);
			serverSelectionServerSelect.setEnabled(false);
			serverSelectionServerPasswordLabel.setEnabled(false);
			serverSelectionServerPassword.setEnabled(false);
			serverSelectionNewserverPortLabel.setEnabled(true);
			serverSelectionNewserverPort.setEnabled(true);
			serverSelectionNewserverPasswordLabel.setEnabled(true);
			serverSelectionNewserverPassword.setEnabled(true);
			
		}
		serverSelectionServerSelect.setValue(settings.getSetting("server-1host") + ":" + settings.getSetting("server-1port"));

		layout[2].createGridBagConstraints(serverIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverHeadline, 1, 0, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionLabel, 1, 1, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionNewserver, 1, 2, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionNewserverPortLabel, 1, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionNewserverPort, 2, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionNewserverPasswordLabel, 1, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionNewserverPassword, 2, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionServer, 1, 5, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionServerSelectLabel, 1, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionServerSelect, 2, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionServerPasswordLabel, 1, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionServerPassword, 2, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(serverSelectionInfo, 1, 8, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		serverSelectionNewserver.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		serverSelectionServer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		tabs[2].add(serverIcon);
		tabs[2].add(serverHeadline);
		tabs[2].add(serverSelectionLabel);
		tabs[2].add(serverSelectionNewserver);
		tabs[2].add(serverSelectionNewserverPasswordLabel);
		tabs[2].add(serverSelectionNewserverPassword);
		tabs[2].add(serverSelectionNewserverPortLabel);
		tabs[2].add(serverSelectionNewserverPort);
		tabs[2].add(serverSelectionServer);
		tabs[2].add(serverSelectionServerSelectLabel);
		tabs[2].add(serverSelectionServerSelect);
		tabs[2].add(serverSelectionServerPasswordLabel);
		tabs[2].add(serverSelectionServerPassword);
		tabs[2].add(serverSelectionInfo);

		tabpane.addTab(settings.getLocalisation("server"), tabs[2]);

		tabpane.setEnabledAt(2, false);

		frame.add(tabpane, BorderLayout.CENTER);
		window.add(frame, BorderLayout.CENTER);

		window.add(frame, BorderLayout.CENTER);

		JPanel bottom = new JPanel();
		okay = new JButton(settings.getLocalisation("ok"));
		okay.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				ok(settings, game, mainWindow, gameArea, owner);
			}
		});
		bottom.add(okay);
		close = new JButton(settings.getLocalisation("cancel"));
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
		bottom.add(close);
		window.add(bottom, BorderLayout.SOUTH);

		validateComponents();
		initializeShortCuts();
		window.setVisible(true);
	}

	private void validateComponents() {
		if (playerSelectionLocal.isSelected()) {
			tabpane.setEnabledAt(2, false);
			tabpane.setEnabledAt(1, true);
			playerSelectionNetworkPlayer1Label.setEnabled(false);
			playerSelectionNetworkPlayer1.setEnabled(false);
			playerSelectionNetworkPlayer2Label.setEnabled(false);
			playerSelectionNetworkPlayer2.setEnabled(false);
			playerSelectionNetworkPlayer3Label.setEnabled(false);
			playerSelectionNetworkPlayer3.setEnabled(false);
			playerSelectionNetworkPlayer4Label.setEnabled(false);
			playerSelectionNetworkPlayer4.setEnabled(false);
			playerSelectionNetworkPlayer5Label.setEnabled(false);
			playerSelectionNetworkPlayer5.setEnabled(false);
		} else if (playerSelectionNetwork.isSelected()) {
			tabpane.setEnabledAt(2, true);
			if (!serverSelectionNewserver.isSelected()) {
				tabpane.setEnabledAt(1, false);
			}
			playerSelectionNetworkPlayer1Label.setEnabled(true);
			playerSelectionNetworkPlayer1.setEnabled(true);
			if (serverSelectionNewserver.isSelected()) {
				playerSelectionNetworkPlayer2Label.setEnabled(true);
				playerSelectionNetworkPlayer2.setEnabled(true);
				playerSelectionNetworkPlayer3Label.setEnabled(true);
				playerSelectionNetworkPlayer3.setEnabled(true);
				playerSelectionNetworkPlayer4Label.setEnabled(true);
				playerSelectionNetworkPlayer4.setEnabled(true);
				playerSelectionNetworkPlayer5Label.setEnabled(true);
				playerSelectionNetworkPlayer5.setEnabled(true);
			} else {
				playerSelectionNetworkPlayer2Label.setEnabled(false);
				playerSelectionNetworkPlayer2.setEnabled(false);
				playerSelectionNetworkPlayer3Label.setEnabled(false);
				playerSelectionNetworkPlayer3.setEnabled(false);
				playerSelectionNetworkPlayer4Label.setEnabled(false);
				playerSelectionNetworkPlayer4.setEnabled(false);
				playerSelectionNetworkPlayer5Label.setEnabled(false);
				playerSelectionNetworkPlayer5.setEnabled(false);
			}
		}
		if (timeSelectionNolimit.isSelected()) {
			timeSelectionLimitTime.setEnabled(false);
		} else if (timeSelectionLimit.isSelected()) {
			timeSelectionLimitTime.setEnabled(true);
		}
		if (serverSelectionNewserver.isSelected()) {
			tabpane.setEnabledAt(1, true);
			serverSelectionServerSelectLabel.setEnabled(false);
			serverSelectionServerSelect.setEnabled(false);
			serverSelectionServerPasswordLabel.setEnabled(false);
			serverSelectionServerPassword.setEnabled(false);
			serverSelectionNewserverPasswordLabel.setEnabled(true);
			serverSelectionNewserverPassword.setEnabled(true);
			serverSelectionNewserverPortLabel.setEnabled(true);
			serverSelectionNewserverPort.setEnabled(true);
		} else if (serverSelectionServer.isSelected()) {
			if (playerSelectionNetwork.isSelected()) {
				tabpane.setEnabledAt(1, false);
			}
			serverSelectionServerSelectLabel.setEnabled(true);
			serverSelectionServerSelect.setEnabled(true);
			serverSelectionServerPasswordLabel.setEnabled(true);
			serverSelectionServerPassword.setEnabled(true);
			serverSelectionNewserverPasswordLabel.setEnabled(false);
			serverSelectionNewserverPassword.setEnabled(false);
			serverSelectionNewserverPortLabel.setEnabled(false);
			serverSelectionNewserverPort.setEnabled(false);
		}
	}

	private void ok(Settings settings, Game game, MainWindow mainWindow, GameComponent gameArea, JFrame owner) {
		if (playerSelectionNetwork.isSelected()) {
			settings.setSetting("multiplayer-taken", "true");
		} else if (playerSelectionLocal.isSelected()) {
			settings.setSetting("multiplayer-taken", "false");
		}
		if (timeSelectionLimit.isSelected()) {
			settings.setSetting("timelimit-taken", "true");
		} else if (timeSelectionNolimit.isSelected()) {
			settings.setSetting("timelimit-taken", "false");
		}
		if (serverSelectionNewserver.isSelected()) {
			settings.setSetting("server-taken", "true");
		} else if (serverSelectionServer.isSelected()) {
			settings.setSetting("server-taken", "false");
		}
		settings.setSetting("multiplayer-0player1", playerSelectionNetworkPlayer1.getText());
		settings.setSetting("multiplayer-0player2", playerSelectionNetworkPlayer2.getText());
		settings.setSetting("multiplayer-0player3", playerSelectionNetworkPlayer3.getText());
		settings.setSetting("multiplayer-0player4", playerSelectionNetworkPlayer4.getText());
		settings.setSetting("multiplayer-0player5", playerSelectionNetworkPlayer5.getText());
		settings.setSetting("timelimit-time", String.valueOf(timeSelectionLimitTime.getValue()));
		settings.setSetting("server-0port", serverSelectionNewserverPort.getValue().split(":")[1]);
		settings.setSetting("server-0password", JPasswordFieldReader.fetchPassword(serverSelectionNewserverPassword));
		settings.setSetting("server-1host", serverSelectionServerSelect.getValue().split(":")[0]);
		settings.setSetting("server-1port", serverSelectionServerSelect.getValue().split(":")[1]);
		settings.setSetting("server-1password", JPasswordFieldReader.fetchPassword(serverSelectionServerPassword));
		settings.saveSettings();

		WaitWindow waitWindow = new WaitWindow(owner, settings);
		game.transmitWaitWindowInstance(waitWindow);
		//waitWindow.start();
		
		game.startNewGame();

		close();
	}

	private void close() {
		window.dispose();
	}

	private void initializeShortCuts() {
		InputMap inputmap = window.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionmap = window.getRootPane().getActionMap();
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "close");
		actionmap.put("close", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				close();
			}
		});
	}
}