/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: SettingsWindow.java - gui
 * Version: 1.0.2
 * Last modified: 2008/10/01 23:45 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.gui.components.AvatarSelector;
import net.reneknipschild.cheesecaskets.gui.components.ColorPicker;
import net.reneknipschild.cheesecaskets.gui.components.HostSelector;
import net.reneknipschild.cheesecaskets.gui.components.IconComboBox;
import net.reneknipschild.cheesecaskets.gui.layouts.ComfortGridBagLayout;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;
import net.reneknipschild.cheesecaskets.tools.ColorConverter;
import net.reneknipschild.cheesecaskets.tools.LookAndFeelSetter;

public class SettingsWindow {
	private boolean restartNecessary;

	private JDialog window;

	private JTabbedPane tabpane;

	private JPanel[] tabs;

	private ComfortGridBagLayout layout[];

	private JLabel localeIcon;

	private JLabel localeHeadline;

	private JLabel localeSelectionLabel;

	private IconComboBox localeSelection;

	private JLabel themeIcon;

	private JLabel themeHeadline;

	private JLabel themeSelectionLabel;

	private ButtonGroup themeSelection;

	private JRadioButton themeSelectionBlack;

	private JRadioButton themeSelectionSilver;

	private JRadioButton themeSelectionGreen;

	private JRadioButton themeSelectionRed;

	private JRadioButton themeSelectionBlue;

	private JLabel avatarIcon;

	private JLabel avatarHeadline;

	private JLabel avatarSelectionLabel;

	private AvatarSelector avatarSelection;

	private JLabel avatarSelectionInfo;

	private JLabel audioIcon;

	private JLabel audioHeadline;

	private JLabel audioSelectionLabel;

	private JCheckBox audioSelectionSound;

	private JCheckBox audioSelectionSoundGUI;

	private JCheckBox audioSelectionSoundGame;

	private JCheckBox audioSelectionSoundGameClick;

	private JCheckBox audioSelectionSoundGameChat;

	private JCheckBox audioSelectionSoundGameBackground;

	private JCheckBox audioSelectionSoundGameEnd;

	private JLabel colorIcon;

	private JLabel colorHeadline;

	private JLabel colorSelectionLabel;

	private JLabel colorSelectionBackgroundLabel;

	private ColorPicker colorSelectionBackground;

	private JLabel colorSelectionInnerBackgroundLabel;

	private ColorPicker colorSelectionInnerBackground;

	private JLabel colorSelectionInnerBorderLabel;

	private ColorPicker colorSelectionInnerBorder;

	private JLabel colorSelectionFieldsetBackgroundLabel;

	private ColorPicker colorSelectionFieldsetBackground;

	private JLabel colorSelectionFieldsetBorderLabel;

	private ColorPicker colorSelectionFieldsetBorder;

	private JLabel colorSelectionFieldBackgroundLabel;

	private ColorPicker colorSelectionFieldBackground;

	private JLabel colorSelectionFieldBorderLabel;

	private ColorPicker colorSelectionFieldBorder;

	private JLabel networkIcon;

	private JLabel networkHeadline;

	private JLabel networkSelectionLabel;

	private ButtonGroup networkSelection;

	private JRadioButton networkSelectionNoproxy;

	private JRadioButton networkSelectionProxy;

	private HostSelector networkProxy;

	private JLabel networkPortInfo;

	public SettingsWindow(final Settings settings, final JFrame owner) {
		window = new JDialog(owner, settings.getLocalisation("settings"), true);
		window.setMinimumSize(new Dimension(475, 500));
		window.setResizable(true);
		window.setLocationRelativeTo(owner);
		window.setIconImage(BitmapLoader.getImage("Icon.png"));
		window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		window.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				close(settings);
			}
		});

		layout = new ComfortGridBagLayout[5];
		layout[0] = new ComfortGridBagLayout();
		layout[1] = new ComfortGridBagLayout();
		layout[2] = new ComfortGridBagLayout();
		layout[3] = new ComfortGridBagLayout();
		layout[4] = new ComfortGridBagLayout();

		JPanel frame = new JPanel(new BorderLayout());
		frame.setBorder(BorderFactory.createEmptyBorder(8, 8, 0, 8));
		tabpane = new JTabbedPane();
		tabs = new JPanel[5];

		tabs[0] = new JPanel(layout[0]);

		localeIcon = new JLabel(BitmapLoader.getIcon("Locale72x72.png"));
		localeHeadline = new JLabel(settings.getLocalisation("language"));
		localeSelectionLabel = new JLabel(settings.getLocalisation("language_type__remark"));
		localeSelection = new IconComboBox();
		themeIcon = new JLabel(BitmapLoader.getIcon("Theme72x72.png"));
		themeHeadline = new JLabel(settings.getLocalisation("theme"));
		themeSelectionLabel = new JLabel(settings.getLocalisation("theme_type__remark"));
		themeSelection = new ButtonGroup();
		themeSelectionBlack = new JRadioButton(settings.getLocalisation("theme_type_black__remark"));
		themeSelectionSilver = new JRadioButton(settings.getLocalisation("theme_type_silver__remark"));
		themeSelectionGreen = new JRadioButton(settings.getLocalisation("theme_type_green__remark"));
		themeSelectionRed = new JRadioButton(settings.getLocalisation("theme_type_red__remark"));
		themeSelectionBlue = new JRadioButton(settings.getLocalisation("theme_type_blue__remark"));

		localeHeadline.setFont(localeHeadline.getFont().deriveFont(Font.BOLD, 24f));
		localeSelection.addItem("English25x15.png", settings.getLocalisation("english"));
		localeSelection.addItem("French25x15.png", settings.getLocalisation("french"));
		localeSelection.addItem("German25x15.png", settings.getLocalisation("german"));
		localeSelection.addItem("Latin25x15.png", settings.getLocalisation("latin"));

		if (settings.getSetting("gui-language").equals("english")) {
			localeSelection.setSelectedIndex(0);
		} else if (settings.getSetting("gui-language").equals("french")) {
			localeSelection.setSelectedIndex(1);
		} else if (settings.getSetting("gui-language").equals("german")) {
			localeSelection.setSelectedIndex(2);
		} else if (settings.getSetting("gui-language").equals("latin")) {
			localeSelection.setSelectedIndex(3);
		}
		themeHeadline.setFont(themeHeadline.getFont().deriveFont(Font.BOLD, 24f));
		if (settings.getSetting("gui-theme").equals("black")) {
			themeSelectionBlack.setSelected(true);
		} else if (settings.getSetting("gui-theme").equals("silver")) {
			themeSelectionSilver.setSelected(true);
		} else if (settings.getSetting("gui-theme").equals("green")) {
			themeSelectionGreen.setSelected(true);
		} else if (settings.getSetting("gui-theme").equals("red")) {
			themeSelectionRed.setSelected(true);
		} else if (settings.getSetting("gui-theme").equals("blue")) {
			themeSelectionBlue.setSelected(true);
		}

		themeSelection.add(themeSelectionBlack);
		themeSelection.add(themeSelectionSilver);
		themeSelection.add(themeSelectionGreen);
		themeSelection.add(themeSelectionRed);
		themeSelection.add(themeSelectionBlue);

		layout[0].createGridBagConstraints(localeIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(localeHeadline, 1, 0, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(localeSelectionLabel, 1, 1, 2, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(localeSelection, 1, 2, 2, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeIcon, 0, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeHeadline, 1, 3, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeSelectionLabel, 1, 4, 2, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeSelectionBlack, 1, 5, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeSelectionSilver, 2, 5, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeSelectionGreen, 1, 6, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeSelectionRed, 2, 6, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[0].createGridBagConstraints(themeSelectionBlue, 1, 7, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		localeSelection.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				restartNecessary = true;
			}
		});

		themeSelectionBlack.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				LookAndFeelSetter.setLook(LookAndFeelSetter.BLACK_LOOK_AND_FEEL);
				SwingUtilities.updateComponentTreeUI(window);
				window.repaint();
			}
		});

		themeSelectionSilver.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				LookAndFeelSetter.setLook(LookAndFeelSetter.SILVER_LOOK_AND_FEEL);
				SwingUtilities.updateComponentTreeUI(window);
				window.repaint();
			}
		});

		themeSelectionGreen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				LookAndFeelSetter.setLook(LookAndFeelSetter.GREEN_LOOK_AND_FEEL);
				SwingUtilities.updateComponentTreeUI(window);
				window.repaint();
			}
		});

		themeSelectionRed.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				LookAndFeelSetter.setLook(LookAndFeelSetter.RED_LOOK_AND_FEEL);
				SwingUtilities.updateComponentTreeUI(window);
				window.repaint();
			}
		});

		themeSelectionBlue.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				LookAndFeelSetter.setLook(LookAndFeelSetter.BLUE_LOOK_AND_FEEL);
				SwingUtilities.updateComponentTreeUI(window);
				window.repaint();
			}
		});

		tabs[0].add(localeIcon);
		tabs[0].add(localeHeadline);
		tabs[0].add(localeSelectionLabel);
		tabs[0].add(localeSelection);
		tabs[0].add(themeIcon);
		tabs[0].add(themeHeadline);
		tabs[0].add(themeSelectionLabel);
		tabs[0].add(themeSelectionBlack);
		tabs[0].add(themeSelectionSilver);
		tabs[0].add(themeSelectionGreen);
		tabs[0].add(themeSelectionRed);
		tabs[0].add(themeSelectionBlue);

		tabpane.addTab(settings.getLocalisation("general"), tabs[0]);

		tabs[1] = new JPanel(layout[1]);

		avatarIcon = new JLabel(BitmapLoader.getIcon("Avatar72x72.png"));
		avatarHeadline = new JLabel(settings.getLocalisation("avatar"));
		avatarSelectionLabel = new JLabel(settings.getLocalisation("avatar_type__remark"));
		avatarSelection = new AvatarSelector(settings, window);
		avatarSelectionInfo = new JLabel("<html><b>" + settings.getLocalisation("note") + ":</b> " + settings.getLocalisation("avatar_type_info__remark") + "</html>");

		avatarHeadline.setFont(avatarHeadline.getFont().deriveFont(Font.BOLD, 24f));

		layout[1].createGridBagConstraints(avatarIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(avatarHeadline, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(avatarSelectionLabel, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(avatarSelection, 1, 2, 1, 1, GridBagConstraints.WEST, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[1].createGridBagConstraints(avatarSelectionInfo, 1, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		tabs[1].add(avatarIcon);
		tabs[1].add(avatarHeadline);
		tabs[1].add(avatarSelectionLabel);
		tabs[1].add(avatarSelection);
		tabs[1].add(avatarSelectionInfo);

		tabpane.addTab(settings.getLocalisation("personal"), tabs[1]);

		tabs[2] = new JPanel(layout[2]);

		audioIcon = new JLabel(BitmapLoader.getIcon("Sound72x72.png"));
		audioHeadline = new JLabel(settings.getLocalisation("sound"));
		audioSelectionLabel = new JLabel(settings.getLocalisation("audio_type__remark"));
		audioSelectionSound = new JCheckBox(settings.getLocalisation("activate_sound"));
		audioSelectionSoundGUI = new JCheckBox(settings.getLocalisation("activate_gui-sounds"));
		audioSelectionSoundGame = new JCheckBox(settings.getLocalisation("activate_game-sounds"));
		audioSelectionSoundGameClick = new JCheckBox(settings.getLocalisation("click_sounds"));
		audioSelectionSoundGameChat = new JCheckBox(settings.getLocalisation("chat_sounds"));
		audioSelectionSoundGameBackground = new JCheckBox(settings.getLocalisation("background_music"));
		audioSelectionSoundGameEnd = new JCheckBox(settings.getLocalisation("game-end_music"));

		audioHeadline.setFont(audioHeadline.getFont().deriveFont(Font.BOLD, 24f));

		if (Boolean.parseBoolean(settings.getSetting("sound-activated"))) {
			audioSelectionSound.setSelected(true);
		}
		if (Boolean.parseBoolean(settings.getSetting("sound-gui"))) {
			audioSelectionSoundGUI.setSelected(true);
		}
		if (Boolean.parseBoolean(settings.getSetting("sound-game"))) {
			audioSelectionSoundGame.setSelected(true);
		}
		if (Boolean.parseBoolean(settings.getSetting("sound-gameclick"))) {
			audioSelectionSoundGameClick.setSelected(true);
		}
		if (Boolean.parseBoolean(settings.getSetting("sound-gamechat"))) {
			audioSelectionSoundGameChat.setSelected(true);
		}
		if (Boolean.parseBoolean(settings.getSetting("sound-gamebackground"))) {
			audioSelectionSoundGameBackground.setSelected(true);
		}
		if (Boolean.parseBoolean(settings.getSetting("sound-gameend"))) {
			audioSelectionSoundGameEnd.setSelected(true);
		}

		layout[2].createGridBagConstraints(audioIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioHeadline, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionLabel, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionSound, 1, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionSoundGUI, 1, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 24, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionSoundGame, 1, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 24, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionSoundGameClick, 1, 5, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 44, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionSoundGameChat, 1, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 44, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionSoundGameBackground, 1, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 44, 4, 4), 0, 0);
		layout[2].createGridBagConstraints(audioSelectionSoundGameEnd, 1, 8, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 44, 4, 4), 0, 0);

		audioSelectionSound.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		audioSelectionSoundGUI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		audioSelectionSoundGame.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		audioSelectionSoundGameClick.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				audioSelectionSoundGameBackground.setSelected(false);
				validateComponents();
			}
		});

		audioSelectionSoundGameChat.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				audioSelectionSoundGameBackground.setSelected(false);
				validateComponents();
			}
		});

		audioSelectionSoundGameBackground.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				audioSelectionSoundGameClick.setSelected(false);
				audioSelectionSoundGameChat.setSelected(false);
				validateComponents();
			}
		});

		audioSelectionSoundGameEnd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				validateComponents();
			}
		});

		tabs[2].add(audioIcon);
		tabs[2].add(audioHeadline);
		tabs[2].add(audioSelectionLabel);
		tabs[2].add(audioSelectionSound);
		tabs[2].add(audioSelectionSoundGUI);
		tabs[2].add(audioSelectionSoundGame);
		tabs[2].add(audioSelectionSoundGameClick);
		tabs[2].add(audioSelectionSoundGameChat);
		tabs[2].add(audioSelectionSoundGameBackground);
		tabs[2].add(audioSelectionSoundGameEnd);

		tabpane.addTab(settings.getLocalisation("audio"), tabs[2]);

		tabs[3] = new JPanel(layout[3]);

		colorIcon = new JLabel(BitmapLoader.getIcon("Colors72x72.png"));
		colorHeadline = new JLabel(settings.getLocalisation("colors"));
		colorSelectionLabel = new JLabel(settings.getLocalisation("color_type__remark"));
		colorSelectionBackgroundLabel = new JLabel(settings.getLocalisation("background"));
		colorSelectionBackground = new ColorPicker(window, settings.getLocalisation("select_color") + "...", ColorConverter.getColorFromHexString(settings.getSetting("colors-background")));
		colorSelectionInnerBackgroundLabel = new JLabel(settings.getLocalisation("game_background"));
		colorSelectionInnerBackground = new ColorPicker(window, settings.getLocalisation("select_color") + "...", ColorConverter.getColorFromHexString(settings.getSetting("colors-gamebackground")));
		colorSelectionInnerBorderLabel = new JLabel(settings.getLocalisation("game_border"));
		colorSelectionInnerBorder = new ColorPicker(window, settings.getLocalisation("select_color") + "...", ColorConverter.getColorFromHexString(settings.getSetting("colors-gameborder")));
		colorSelectionFieldsetBackgroundLabel = new JLabel(settings.getLocalisation("fieldset_background"));
		colorSelectionFieldsetBackground = new ColorPicker(window, settings.getLocalisation("select_color") + "...", ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldsetbackground")));
		colorSelectionFieldsetBorderLabel = new JLabel(settings.getLocalisation("fieldset_border"));
		colorSelectionFieldsetBorder = new ColorPicker(window, settings.getLocalisation("select_color") + "...", ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldsetborder")));
		colorSelectionFieldBackgroundLabel = new JLabel(settings.getLocalisation("field_background"));
		colorSelectionFieldBackground = new ColorPicker(window, settings.getLocalisation("select_color") + "...", ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldbackground")));
		colorSelectionFieldBorderLabel = new JLabel(settings.getLocalisation("field_border"));
		colorSelectionFieldBorder = new ColorPicker(window, settings.getLocalisation("select_color") + "...", ColorConverter.getColorFromHexString(settings.getSetting("colors-fieldborder")));

		colorHeadline.setFont(colorHeadline.getFont().deriveFont(Font.BOLD, 24f));

		layout[3].createGridBagConstraints(colorIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorHeadline, 1, 0, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionLabel, 1, 1, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionBackgroundLabel, 1, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionBackground, 2, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionInnerBackgroundLabel, 1, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionInnerBackground, 2, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionInnerBorderLabel, 1, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionInnerBorder, 2, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldsetBackgroundLabel, 1, 5, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldsetBackground, 2, 5, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldsetBorderLabel, 1, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldsetBorder, 2, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldBackgroundLabel, 1, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldBackground, 2, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldBorderLabel, 1, 8, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[3].createGridBagConstraints(colorSelectionFieldBorder, 2, 8, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		tabs[3].add(colorIcon);
		tabs[3].add(colorHeadline);
		tabs[3].add(colorSelectionLabel);
		tabs[3].add(colorSelectionBackgroundLabel);
		tabs[3].add(colorSelectionBackground);
		tabs[3].add(colorSelectionInnerBackgroundLabel);
		tabs[3].add(colorSelectionInnerBackground);
		tabs[3].add(colorSelectionInnerBorderLabel);
		tabs[3].add(colorSelectionInnerBorder);
		tabs[3].add(colorSelectionFieldsetBackgroundLabel);
		tabs[3].add(colorSelectionFieldsetBackground);
		tabs[3].add(colorSelectionFieldsetBorderLabel);
		tabs[3].add(colorSelectionFieldsetBorder);
		tabs[3].add(colorSelectionFieldBackgroundLabel);
		tabs[3].add(colorSelectionFieldBackground);
		tabs[3].add(colorSelectionFieldBorderLabel);
		tabs[3].add(colorSelectionFieldBorder);

		tabpane.addTab(settings.getLocalisation("game"), tabs[3]);

		tabs[4] = new JPanel(layout[4]);

		networkIcon = new JLabel(BitmapLoader.getIcon("Network72x72.png"));
		networkHeadline = new JLabel(settings.getLocalisation("proxy"));
		networkSelectionLabel = new JLabel(settings.getLocalisation("proxy_type__remark"));
		networkSelection = new ButtonGroup();
		networkSelectionNoproxy = new JRadioButton(settings.getLocalisation("proxy_type_noproxy__remark"));
		networkSelectionProxy = new JRadioButton(settings.getLocalisation("proxy_type_proxy__remark"));
		networkProxy = new HostSelector(true, settings.getLocalisation("proxy") + ":", true, settings.getLocalisation("port") + ":", HostSelector.HOST_AND_PORT);
		networkPortInfo = new JLabel("<html><b>" + settings.getLocalisation("note") + ":</b> " + settings.getLocalisation("proxy_type_info__remark") + "</html>");

		networkHeadline.setFont(networkHeadline.getFont().deriveFont(Font.BOLD, 24f));
		networkSelection.add(networkSelectionNoproxy);
		networkSelection.add(networkSelectionProxy);
		if (Boolean.parseBoolean(settings.getSetting("proxy-taken"))) {
			networkSelectionProxy.setSelected(true);
			networkProxy.setEnabled(true);
		} else {
			networkSelectionNoproxy.setSelected(true);
			networkProxy.setEnabled(false);
		}
		networkProxy.setValue(settings.getSetting("proxy-0host") + ":" + settings.getSetting("proxy-0port"));

		layout[4].createGridBagConstraints(networkIcon, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 0, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[4].createGridBagConstraints(networkHeadline, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[4].createGridBagConstraints(networkSelectionLabel, 1, 1, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[4].createGridBagConstraints(networkSelectionNoproxy, 1, 2, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[4].createGridBagConstraints(networkSelectionProxy, 1, 3, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[4].createGridBagConstraints(networkProxy, 1, 4, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout[4].createGridBagConstraints(networkPortInfo, 1, 5, 1, 1, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);

		networkSelectionNoproxy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				networkProxy.setEnabled(false);
			}
		});

		networkSelectionProxy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				networkProxy.setEnabled(true);
			}
		});

		tabs[4].add(networkIcon);
		tabs[4].add(networkHeadline);
		tabs[4].add(networkSelectionLabel);
		tabs[4].add(networkSelectionNoproxy);
		tabs[4].add(networkSelectionProxy);
		tabs[4].add(networkProxy);
		tabs[4].add(networkPortInfo);

		tabpane.addTab(settings.getLocalisation("network"), tabs[4]);

		frame.add(tabpane, BorderLayout.CENTER);
		window.add(frame, BorderLayout.CENTER);

		JPanel bottom = new JPanel();
		JButton close = new JButton(settings.getLocalisation("close"));
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				close(settings);
			}
		});
		bottom.add(close);
		window.add(bottom, BorderLayout.SOUTH);

		validateComponents();
		initializeShortCuts(settings);
		window.setVisible(true);
	}

	private void validateComponents() {
		if (audioSelectionSound.isSelected()) {
			audioSelectionSoundGUI.setEnabled(true);
			audioSelectionSoundGame.setEnabled(true);
			if (audioSelectionSoundGame.isSelected()) {
				audioSelectionSoundGameClick.setEnabled(true);
				audioSelectionSoundGameChat.setEnabled(true);
				audioSelectionSoundGameBackground.setEnabled(true);
				audioSelectionSoundGameEnd.setEnabled(true);
			} else {
				audioSelectionSoundGameClick.setEnabled(false);
				audioSelectionSoundGameChat.setEnabled(false);
				audioSelectionSoundGameBackground.setEnabled(false);
				audioSelectionSoundGameEnd.setEnabled(false);
			}
		} else {
			audioSelectionSoundGUI.setEnabled(false);
			audioSelectionSoundGame.setEnabled(false);
			audioSelectionSoundGameClick.setEnabled(false);
			audioSelectionSoundGameChat.setEnabled(false);
			audioSelectionSoundGameBackground.setEnabled(false);
			audioSelectionSoundGameEnd.setEnabled(false);
		}
	}

	private void close(Settings settings) {
		if (restartNecessary) {
			JOptionPane.showMessageDialog(window, settings.getLocalisation("restart_necessary__remark"), settings.getLocalisation("warning"), JOptionPane.WARNING_MESSAGE);
		}
		window.dispose();
		switch (localeSelection.getSelectedIndex()) {
		case 0:
			settings.setSetting("gui-language", "english");
			break;
		case 1:
			settings.setSetting("gui-language", "french");
			break;
		case 2:
			settings.setSetting("gui-language", "german");
			break;
		case 3:
			settings.setSetting("gui-language", "latin");
			break;
		}
		if (themeSelectionBlack.isSelected()) {
			settings.setSetting("gui-theme", "black");
		} else if (themeSelectionSilver.isSelected()) {
			settings.setSetting("gui-theme", "silver");
		} else if (themeSelectionGreen.isSelected()) {
			settings.setSetting("gui-theme", "green");
		} else if (themeSelectionRed.isSelected()) {
			settings.setSetting("gui-theme", "red");
		} else if (themeSelectionBlue.isSelected()) {
			settings.setSetting("gui-theme", "blue");
		}
		settings.setSetting("sound-activated", String.valueOf(audioSelectionSound.isSelected()));
		settings.setSetting("sound-gui", String.valueOf(audioSelectionSoundGUI.isSelected()));
		if (audioSelectionSound.isSelected()) {
			LookAndFeelSetter.setSound(audioSelectionSoundGUI.isSelected());
		} else {
			LookAndFeelSetter.setSound(false);
		}
		settings.setSetting("sound-game", String.valueOf(audioSelectionSoundGame.isSelected()));
		settings.setSetting("sound-gameclick", String.valueOf(audioSelectionSoundGameClick.isSelected()));
		settings.setSetting("sound-gamechat", String.valueOf(audioSelectionSoundGameChat.isSelected()));
		settings.setSetting("sound-gamebackground", String.valueOf(audioSelectionSoundGameBackground.isSelected()));
		settings.setSetting("sound-gameend", String.valueOf(audioSelectionSoundGameEnd.isSelected()));
		settings.setSetting("colors-background", ColorConverter.getHexStringFromColor(colorSelectionBackground.getColor()));
		settings.setSetting("colors-gamebackground", ColorConverter.getHexStringFromColor(colorSelectionInnerBackground.getColor()));
		settings.setSetting("colors-gameborder", ColorConverter.getHexStringFromColor(colorSelectionInnerBorder.getColor()));
		settings.setSetting("colors-fieldsetbackground", ColorConverter.getHexStringFromColor(colorSelectionFieldsetBackground.getColor()));
		settings.setSetting("colors-fieldsetborder", ColorConverter.getHexStringFromColor(colorSelectionFieldsetBorder.getColor()));
		settings.setSetting("colors-fieldbackground", ColorConverter.getHexStringFromColor(colorSelectionFieldBackground.getColor()));
		settings.setSetting("colors-fieldborder", ColorConverter.getHexStringFromColor(colorSelectionFieldBorder.getColor()));
		if (networkSelectionNoproxy.isSelected()) {
			settings.setSetting("proxy-taken", "false");
		} else if (networkSelectionProxy.isSelected()) {
			settings.setSetting("proxy-taken", "true");
		}
		settings.setSetting("proxy-0host", networkProxy.getValue().split(":")[0]);
		settings.setSetting("proxy-0port", networkProxy.getValue().split(":")[1]);
		settings.saveSettings();
	}

	private void initializeShortCuts(final Settings settings) {
		InputMap inputmap = window.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionmap = window.getRootPane().getActionMap();
		inputmap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "close");
		actionmap.put("close", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent event) {
				close(settings);
			}
		});
	}
}