/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: SidebarComponent.java - gui
 * Version: 1.0.1
 * Last modified: 2008/10/02 16:51 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.gui.layouts.ComfortGridBagLayout;

public class SidebarComponent extends JPanel {
	private static final long serialVersionUID = 1L;

	private Settings settings;

	private JPanel frame;

	private JLabel sidebar;

	private JLabel headline;

	private JLabel timeElapsedLabel;

	private JLabel timeElapsed;

	private JLabel timeRemainingLabel;

	private JLabel timeRemaining;

	private JLabel playersLabel;

	private JPanel playersPanel;

	private JList players;

	public SidebarComponent(Settings settings) {
		this.settings = settings;

		setLayout(new BorderLayout());
		ComfortGridBagLayout layout = new ComfortGridBagLayout();
		frame = new JPanel(layout);
		frame.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));

		sidebar = new JLabel("<html><b>" + settings.getLocalisation("sidebar") + "</b></html>");
		headline = new JLabel("<html><font size=6><b>" + settings.getLocalisation("no_game") + "</b></font></html>");
		timeElapsedLabel = new JLabel("<html><b>" + settings.getLocalisation("time_elapsed") + ":</b></html>");
		timeElapsed = new JLabel("<" + settings.getLocalisation("no_game") + ">");
		timeRemainingLabel = new JLabel("<html><b>" + settings.getLocalisation("time_remaining") + ":</b></html>");
		timeRemaining = new JLabel("<" + settings.getLocalisation("no_game") + ">");
		playersLabel = new JLabel("<html><b>" + settings.getLocalisation("players") + ":</b></html>");
		playersPanel = new JPanel(new BorderLayout());
		String[] list = { "<" + settings.getLocalisation("no_game") + ">" };
		players = new JList(list);

		playersPanel.add(new JScrollPane(players, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);

		layout.createGridBagConstraints(sidebar, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout.createGridBagConstraints(headline, 0, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(4, 4, 4, 4), 0, 0);
		layout.createGridBagConstraints(timeElapsedLabel, 0, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(8, 4, 0, 4), 0, 0);
		layout.createGridBagConstraints(timeElapsed, 0, 3, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(0, 4, 4, 4), 0, 0);
		layout.createGridBagConstraints(timeRemainingLabel, 0, 4, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(8, 4, 0, 4), 0, 0);
		layout.createGridBagConstraints(timeRemaining, 0, 5, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(0, 4, 4, 4), 0, 0);
		layout.createGridBagConstraints(playersLabel, 0, 6, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, new Insets(8, 4, 0, 4), 0, 0);
		layout.createGridBagConstraints(playersPanel, 0, 7, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 1, 1, new Insets(0, 4, 4, 4), 0, 0);

		frame.add(sidebar);
		frame.add(headline);
		frame.add(timeElapsedLabel);
		frame.add(timeElapsed);
		frame.add(timeRemainingLabel);
		frame.add(timeRemaining);
		frame.add(playersLabel);
		frame.add(playersPanel);

		this.add(frame, BorderLayout.CENTER);
	}

	public void switchGameMode(boolean on) {
		if (on) {
			headline.setText("<html><font size=6><b>" + settings.getLocalisation("current_game") + "</b></font></html>");
			timeElapsed.setText("-:--");
			timeRemaining.setText("-:--");
			String[] list = { null };
			players.setListData(list);
		} else {
			headline.setText("<html><font size=6><b>" + settings.getLocalisation("no_game") + "</b></font></html>");
			timeElapsed.setText("<" + settings.getLocalisation("no_game") + ">");
			timeRemaining.setText("<" + settings.getLocalisation("no_game") + ">");
			String[] list = { "<" + settings.getLocalisation("no_game") + ">" };
			players.setListData(list);
		}
	}

	public void updateTimeElapsed(String newTime) {
		timeElapsed.setText(newTime);
	}

	public void updateTimeRemaining(String newTime) {
		timeRemaining.setText(newTime);
	}

	public void updatePlayersList(String[] newPlayers) {
		players.setListData(newPlayers);
	}
}