/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: AvatarSelector.java - gui/components
 * Version: 1.0.0
 * Last modified: 2008/07/23 23:44 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.activation.MimetypesFileTypeMap;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

import net.reneknipschild.cheesecaskets.core.Settings;
import net.reneknipschild.cheesecaskets.tools.BitmapLoader;

public class AvatarSelector extends JButton {
	private static final long serialVersionUID = 1L;

	private JFileChooser filechooser;

	private FileNameExtensionFilter filefilter;

	public AvatarSelector(final Settings settings, final JDialog owner) {
		setLayout(new BorderLayout());
		if (!Boolean.parseBoolean(settings.getSetting("avatar-taken"))) {
			setIcon(BitmapLoader.getIcon("User128x128.png"));
		} else {
			setIcon(settings.getAvatar());
		}
		filechooser = new JFileChooser();
		filefilter = new FileNameExtensionFilter(settings.getLocalisation("images"), "png", "jpg", "jpeg", "gif");
		filechooser.setFileFilter(filefilter);
		addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				boolean loop;
				do {
					loop = false;
					if (filechooser.showOpenDialog(owner) == JFileChooser.APPROVE_OPTION) {
						MimetypesFileTypeMap mimeTypesMap = new MimetypesFileTypeMap();
						mimeTypesMap.addMimeTypes("image/png png");
						String contentType = mimeTypesMap.getContentType(filechooser.getSelectedFile());
						if (contentType.equals("image/png") || contentType.equals("image/jpeg") || contentType.equals("image/gif")) {
							settings.setAvatar(filechooser.getSelectedFile().getAbsolutePath());
							settings.setSetting("avatar-taken", "true");
							setIcon(settings.getAvatar());
						} else {
							JOptionPane.showMessageDialog(owner, settings.getLocalisation("avatar_type_error__remark"), settings.getLocalisation("error"), JOptionPane.ERROR_MESSAGE);
							loop = true;
						}
						updateUI();
					}
				} while (loop);
			}
		});
	}

	public Dimension getPreferredSize() {
		return new Dimension(128, 128);
	}
}