/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: ColorPicker.java - gui/components
 * Version: 0.0.1
 * Last modified: 2008/07/09 00:03:00 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;

public class ColorPicker extends JButton {
	private static final long serialVersionUID = 1L;

	private Color newColor;

	private Color currentColor;

	public ColorPicker(final JDialog owner, final String title, final Color initialColor) {
		currentColor = initialColor;
		setIcon(new ColorIcon());
		setMinimumSize(new Dimension(32, 32));
		addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				newColor = JColorChooser.showDialog(owner, title, currentColor);
				if (newColor != null) {
					ColorPicker.this.setColor(newColor);
				}
			}
		});
	}

	public void setColor(Color newColor) {
		currentColor = newColor;
		this.repaint();
	}

	public Color getColor() {
		return currentColor;
	}

	public Dimension getPreferredSize() {
		return getMinimumSize();
	}

	class ColorIcon implements Icon {
		public int getIconHeight() {
			return ColorPicker.this.getPreferredSize().height;
		}

		public int getIconWidth() {
			return ColorPicker.this.getPreferredSize().width;
		}

		public void paintIcon(Component component, Graphics graphics, int x, int y) {
			graphics.setColor(currentColor);
			graphics.fillRect(x + 5, y + 5, getIconWidth() - 10, getIconHeight() - 10);
		}
	}
}