/*
 * ##################################################
 * #                                                #
 * # CheeseCaskets                                  #
 * # An extended TicTacToe game                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                          rk@reneknipschild.net #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: HostSelector.java - gui/components
 * Version: 1.0.0
 * Last modified: 2008/07/23 23:44 CEST
 * Author: DeeDee0815
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package net.reneknipschild.cheesecaskets.gui.components;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;

import net.reneknipschild.cheesecaskets.gui.layouts.ComfortGridBagLayout;

public class HostSelector extends JPanel {
	private static final long serialVersionUID = 1L;

	private Insets insetsForHostLabel;

	private Insets insetsForHost;

	private Insets insetsForPortLabel;

	private Insets insetsForPort;

	private JLabel hostLabel;

	private JTextField host;

	private JLabel portLabel;

	private JSpinner port;

	private String value;

	private static final String IP_REGEX = "^(?:(?:[0-2](?:(?<=[01])[0-9]{2}|(?<=2)[0-5](?:(?<=[0-4])[0-9]|(?<=5)[0-5]))|[0-9]{1,2})\\.){3}(?:[0-2](?:(?<=[01])[0-9]{2}|(?<=2)[0-5](?:(?<=[0-4])[0-9]|(?<=5)[0-5]))|[0-9]{1,2})$";

	public static final int HOST_AND_PORT = 0;

	public static final int HOST_ONLY = 1;

	public static final int PORT_ONLY = 2;

	public HostSelector(boolean hostLabeled, String hostText, boolean portLabeled, String portText, int mode) {
		ComfortGridBagLayout layout = new ComfortGridBagLayout();
		setLayout(layout);

		insetsForHostLabel = new Insets(0, 0, 0, 0);
		insetsForHost = new Insets(0, 0, 0, 0);
		insetsForPortLabel = new Insets(0, 0, 0, 0);
		insetsForPort = new Insets(0, 0, 0, 0);
		
		hostLabel = new JLabel(hostText);
		host = new JTextField("0.0.0.0");
		portLabel = new JLabel(portText);
		port = new JSpinner(new SpinnerNumberModel(0, 0, 65535, 1));
		
		switch (mode) {
		case 0:
			insetsForHostLabel = new Insets(0, 0, 0, 4);
			if (hostLabeled) {
				insetsForHost = new Insets(0, 4, 0, 4);
			} else {
				insetsForHost = new Insets(0, 0, 0, 4);
				hostLabel.setVisible(false);
			}
			if (!portLabeled) {
				portLabel.setVisible(false);
			}
			insetsForPortLabel = new Insets(0, 4, 0, 4);
			insetsForPort = new Insets(0, 4, 0, 0);
			break;
		case 1:
			insetsForHostLabel = new Insets(0, 0, 0, 4);
			if (hostLabeled) {
				insetsForHost = new Insets(0, 4, 0, 0);
			} else {
				insetsForHost = new Insets(0, 0, 0, 0);
			}
			portLabel.setVisible(false);
			port.setVisible(false);
			break;
		case 2:
			insetsForPortLabel = new Insets(0, 4, 0, 4);
			insetsForPort = new Insets(0, 4, 0, 0);
			hostLabel.setVisible(false);
			host.setVisible(false);
			break;
		}

		layout.createGridBagConstraints(hostLabel, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 0, 0, insetsForHostLabel, 0, 0);
		layout.createGridBagConstraints(host, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 1000000, 0, insetsForHost, 0, 0);
		layout.createGridBagConstraints(portLabel, 2, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 0, 0, insetsForPortLabel, 0, 0);
		layout.createGridBagConstraints(port, 3, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 1, 0, insetsForPort, 0, 0);

		value = "";
		host.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent event) {
				value = host.getText();
			}

			public void focusLost(FocusEvent event) {
				if (!host.getText().matches(IP_REGEX)) {
					host.setText(value);
				}
			}
		});

		this.add(hostLabel);
		this.add(host);
		this.add(portLabel);
		this.add(port);
	}

	public void setValue(String value) {
		String[] text = value.split(":");
		host.setText(text[0]);
		port.setValue(Integer.parseInt(text[1]));
	}

	public String getValue() {
		String host = this.host.getText();
		String port = String.valueOf(this.port.getValue());
		return host + ":" + port;
	}

	public void setEnabled(boolean enabled) {
		if (enabled) {
			hostLabel.setEnabled(true);
			host.setEnabled(true);
			portLabel.setEnabled(true);
			port.setEnabled(true);
		} else {
			hostLabel.setEnabled(false);
			host.setEnabled(false);
			portLabel.setEnabled(false);
			port.setEnabled(false);
		}
	}
}